# coding:utf-8

"""
    CSS color utilities
    ~~~~~~~~~~~~~~~~~~~
    
    
    some parts are borrowed from colorname sourcecode (GPL v2):
        http://code.foosel.org/colorname (
        by: Philippe 'demod' Neumann and Gina 'foosel' Häußge
        
    :copyleft: 2009-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import re
import math


CSS_RE = re.compile(r'#([a-f0-9]{6})[\s;]', re.IGNORECASE)
CSS_CONVERT_RE = re.compile(r'#([a-f0-9])([a-f0-9])([a-f0-9]) *;', re.IGNORECASE)


class ColorVector(tuple):
    def __sub__(self, v):
        return (self[0] - v[0], self[1] - v[1], self[2] - v[2])

COLOR_NAMES = { # CSS3 names, http://www.w3.org/TR/css3-color/
    'indigo': ColorVector((75, 0, 130)),
    'gold': ColorVector((255, 215, 0)),
    'hotpink': ColorVector((255, 105, 180)),
    'firebrick': ColorVector((178, 34, 34)),
    'indianred': ColorVector((205, 92, 92)),
    'yellow': ColorVector((255, 255, 0)),
    'mistyrose': ColorVector((255, 228, 225)),
    'darkolivegreen': ColorVector((85, 107, 47)),
    'olive': ColorVector((128, 128, 0)),
    'darkseagreen': ColorVector((143, 188, 143)),
    'pink': ColorVector((255, 192, 203)),
    'tomato': ColorVector((255, 99, 71)),
    'lightcoral': ColorVector((240, 128, 128)),
    'orangered': ColorVector((255, 69, 0)),
    'navajowhite': ColorVector((255, 222, 173)),
    'lime': ColorVector((0, 255, 0)),
    'palegreen': ColorVector((152, 251, 152)),
    'darkslategrey': ColorVector((47, 79, 79)),
    'greenyellow': ColorVector((173, 255, 47)),
    'burlywood': ColorVector((222, 184, 135)),
    'seashell': ColorVector((255, 245, 238)),
    'mediumspringgreen': ColorVector((0, 250, 154)),
    'fuchsia': ColorVector((255, 0, 255)),
    'papayawhip': ColorVector((255, 239, 213)),
    'blanchedalmond': ColorVector((255, 235, 205)),
    'chartreuse': ColorVector((127, 255, 0)),
    'dimgray': ColorVector((105, 105, 105)),
    'black': ColorVector((0, 0, 0)),
    'peachpuff': ColorVector((255, 218, 185)),
    'springgreen': ColorVector((0, 255, 127)),
    'aquamarine': ColorVector((127, 255, 212)),
    'white': ColorVector((255, 255, 255)),
    'orange': ColorVector((255, 165, 0)),
    'lightsalmon': ColorVector((255, 160, 122)),
    'darkslategray': ColorVector((47, 79, 79)),
    'brown': ColorVector((165, 42, 42)),
    'ivory': ColorVector((255, 255, 240)),
    'dodgerblue': ColorVector((30, 144, 255)),
    'peru': ColorVector((205, 133, 63)),
    'lawngreen': ColorVector((124, 252, 0)),
    'chocolate': ColorVector((210, 105, 30)),
    'crimson': ColorVector((220, 20, 60)),
    'forestgreen': ColorVector((34, 139, 34)),
    'darkgrey': ColorVector((169, 169, 169)),
    'lightseagreen': ColorVector((32, 178, 170)),
    'cyan': ColorVector((0, 255, 255)),
    'mintcream': ColorVector((245, 255, 250)),
    'silver': ColorVector((192, 192, 192)),
    'antiquewhite': ColorVector((250, 235, 215)),
    'mediumorchid': ColorVector((186, 85, 211)),
    'skyblue': ColorVector((135, 206, 235)),
    'gray': ColorVector((128, 128, 128)),
    'darkturquoise': ColorVector((0, 206, 209)),
    'goldenrod': ColorVector((218, 165, 32)),
    'darkgreen': ColorVector((0, 100, 0)),
    'floralwhite': ColorVector((255, 250, 240)),
    'darkviolet': ColorVector((148, 0, 211)),
    'darkgray': ColorVector((169, 169, 169)),
    'moccasin': ColorVector((255, 228, 181)),
    'saddlebrown': ColorVector((139, 69, 19)),
    'grey': ColorVector((128, 128, 128)),
    'darkslateblue': ColorVector((72, 61, 139)),
    'lightskyblue': ColorVector((135, 206, 250)),
    'lightpink': ColorVector((255, 182, 193)),
    'mediumvioletred': ColorVector((199, 21, 133)),
    'slategrey': ColorVector((112, 128, 144)),
    'red': ColorVector((255, 0, 0)),
    'deeppink': ColorVector((255, 20, 147)),
    'limegreen': ColorVector((50, 205, 50)),
    'darkmagenta': ColorVector((139, 0, 139)),
    'palegoldenrod': ColorVector((238, 232, 170)),
    'plum': ColorVector((221, 160, 221)),
    'turquoise': ColorVector((64, 224, 208)),
    'lightgrey': ColorVector((211, 211, 211)),
    'lightgoldenrodyellow': ColorVector((250, 250, 210)),
    'darkgoldenrod': ColorVector((184, 134, 11)),
    'lavender': ColorVector((230, 230, 250)),
    'maroon': ColorVector((128, 0, 0)),
    'yellowgreen': ColorVector((154, 205, 50)),
    'sandybrown': ColorVector((244, 164, 96)),
    'thistle': ColorVector((216, 191, 216)),
    'violet': ColorVector((238, 130, 238)),
    'navy': ColorVector((0, 0, 128)),
    'magenta': ColorVector((255, 0, 255)),
    'dimgrey': ColorVector((105, 105, 105)),
    'tan': ColorVector((210, 180, 140)),
    'rosybrown': ColorVector((188, 143, 143)),
    'olivedrab': ColorVector((107, 142, 35)),
    'blue': ColorVector((0, 0, 255)),
    'lightblue': ColorVector((173, 216, 230)),
    'ghostwhite': ColorVector((248, 248, 255)),
    'honeydew': ColorVector((240, 255, 240)),
    'cornflowerblue': ColorVector((100, 149, 237)),
    'slateblue': ColorVector((106, 90, 205)),
    'linen': ColorVector((250, 240, 230)),
    'darkblue': ColorVector((0, 0, 139)),
    'powderblue': ColorVector((176, 224, 230)),
    'seagreen': ColorVector((46, 139, 87)),
    'darkkhaki': ColorVector((189, 183, 107)),
    'snow': ColorVector((255, 250, 250)),
    'sienna': ColorVector((160, 82, 45)),
    'mediumblue': ColorVector((0, 0, 205)),
    'royalblue': ColorVector((65, 105, 225)),
    'lightcyan': ColorVector((224, 255, 255)),
    'green': ColorVector((0, 128, 0)),
    'mediumpurple': ColorVector((147, 112, 216)),
    'midnightblue': ColorVector((25, 25, 112)),
    'cornsilk': ColorVector((255, 248, 220)),
    'paleturquoise': ColorVector((175, 238, 238)),
    'bisque': ColorVector((255, 228, 196)),
    'slategray': ColorVector((112, 128, 144)),
    'darkcyan': ColorVector((0, 139, 139)),
    'khaki': ColorVector((240, 230, 140)),
    'wheat': ColorVector((245, 222, 179)),
    'teal': ColorVector((0, 128, 128)),
    'darkorchid': ColorVector((153, 50, 204)),
    'deepskyblue': ColorVector((0, 191, 255)),
    'salmon': ColorVector((250, 128, 114)),
    'darkred': ColorVector((139, 0, 0)),
    'steelblue': ColorVector((70, 130, 180)),
    'palevioletred': ColorVector((216, 112, 147)),
    'lightslategray': ColorVector((119, 136, 153)),
    'aliceblue': ColorVector((240, 248, 255)),
    'lightslategrey': ColorVector((119, 136, 153)),
    'lightgreen': ColorVector((144, 238, 144)),
    'orchid': ColorVector((218, 112, 214)),
    'gainsboro': ColorVector((220, 220, 220)),
    'mediumseagreen': ColorVector((60, 179, 113)),
    'lightgray': ColorVector((211, 211, 211)),
    'mediumturquoise': ColorVector((72, 209, 204)),
    'lemonchiffon': ColorVector((255, 250, 205)),
    'cadetblue': ColorVector((95, 158, 160)),
    'lightyellow': ColorVector((255, 255, 224)),
    'lavenderblush': ColorVector((255, 240, 245)),
    'coral': ColorVector((255, 127, 80)),
    'purple': ColorVector((128, 0, 128)),
    'aqua': ColorVector((0, 255, 255)),
    'whitesmoke': ColorVector((245, 245, 245)),
    'mediumslateblue': ColorVector((123, 104, 238)),
    'darkorange': ColorVector((255, 140, 0)),
    'mediumaquamarine': ColorVector((102, 205, 170)),
    'darksalmon': ColorVector((233, 150, 122)),
    'beige': ColorVector((245, 245, 220)),
    'blueviolet': ColorVector((138, 43, 226)),
    'azure': ColorVector((240, 255, 255)),
    'lightsteelblue': ColorVector((176, 196, 222)),
    'oldlace': ColorVector((253, 245, 230)),
}


def hypot(a, b):
    return math.sqrt(a * a + b * b)


def calc_distance(a, b):
    """
    @return: the euclidean distance between the vectors "a" and "b"
    @param a,b: ColorVector objects
    
    >>> calc_distance(ColorVector((255, 255, 255)), ColorVector((255, 255, 255)))
    0.0
    >>> calc_distance(ColorVector((255, 255, 255)), ColorVector((255, 255, 128)))
    127.0
    >>> calc_distance(ColorVector((255, 255, 255)), ColorVector((255, 0, 255)))
    255.0
    """
    return reduce(hypot, a - b)


def hex_string2colorvector(hex_string):
    r = int(hex_string[:2], 16)
    g = int(hex_string[2:4], 16)
    b = int(hex_string[4:6], 16)
    return ColorVector((r, g, b))


def hex2color_names(hex_string):
    """
    >>> sorted(hex2color_names("FF0000"))[0]
    (0.0, 'red')
    >>> sorted(hex2color_names("faebd7"))[0]
    (0.0, 'antiquewhite')
    >>> sorted(hex2color_names("faebd4"))[0]
    (3.0, 'antiquewhite')
    """
    sourcecolor = hex_string2colorvector(hex_string)
    result = []
    for color_name, colorvector in COLOR_NAMES.iteritems():
        distance = calc_distance(sourcecolor, colorvector)
        result.append((distance, color_name))
    return result


def hex2color_name(hex_string):
    """
    >>> hex2color_name("7b68ee")
    (0.0, 'mediumslateblue')
    >>> hex2color_name("f5f5f5")
    (0.0, 'whitesmoke')
    >>> hex2color_name("f5f3f5")
    (2.0, 'whitesmoke')
    """
    sourcecolor = hex_string2colorvector(hex_string)
    best_distance = 256
    best_name = "unknown"
    for color_name, colorvector in COLOR_NAMES.iteritems():
        distance = calc_distance(sourcecolor, colorvector)
        if distance < best_distance:
            best_distance = distance
            best_name = color_name
            if distance == 0.0:
                break

    return (best_distance, best_name)


#------------------------------------------------------------------------------


CSS_RE2 = re.compile(r'#([a-f0-9]{3,6}) *;', re.IGNORECASE)

def unify_spelling(content):
    """
    unify the 'spelling' of all existing css color values.
    
    >>> unify_spelling("color1: #f00; color2: #Ff0000;")
    ('color1: #ff0000; color2: #ff0000; color3: #aabbcc;', set(['ff0000']))
    
    >>> unify_spelling("color: #aAbBcC  ;")
    ('color: #aabbcc;', set(['aabbcc']))
    """
    existing_values = set()
    def callback(matchobj):
        css_value = matchobj.group(1)
        if len(css_value) == 3:
            # convert 3 length css color values to 6 length
            css_value = css_value[0] + css_value[0] + css_value[1] + css_value[1] + css_value[2] + css_value[2]
        css_value = css_value.lower()

        existing_values.add(css_value)
        return "#%s;" % css_value

    new_content = CSS_RE2.sub(callback, content)
    return new_content, existing_values




def filter_content(content):
    """
    Skip all lines, start with .pygments - Used in update routine.
    Note: result is not valid CSS!
    """
    result = []
    for line in content.splitlines():
        line = line.strip()
        if line and not line.startswith(".pygments"):
            result.append(line)
    return "\n".join(result)


def unique_color_name(existing_colors, hex_string):
    """   
    >>> unique_color_name((), "ffff00")
    'yellow'
    
    >>> unique_color_name(('yellow',), "ffff00")
    'yellow_2'
    
    >>> unique_color_name((), "0000f5")
    'blue_d10'
    
    >>> unique_color_name(('blue_d10',), "0000f5")
    'blue_d10_2'
    """
    distance, color_name = hex2color_name(hex_string)
    if distance > 5.0:
        # add distance if color values are too different
        color_name += "_d%i" % round(distance)

    if color_name in existing_colors:
        # name exist -> make unique by adding a number
        for no in xrange(2, 1000):
            # get a new color name, witch not exist yet.
            test_name = "%s_%s" % (color_name, no)
            if test_name not in existing_colors:
                color_name = test_name
                break
    return color_name


def convert_3to6_colors(content):
    """
    Convert all 3 length css color values to 6 length
    
    >>> convert_3to6_colors("#f00; #aabbcc;")
    '#ff0000; #aabbcc;'
    """
    new_content = CSS_CONVERT_RE.sub("#\g<1>\g<1>\g<2>\g<2>\g<3>\g<3>;", content)
    return new_content


def findall_color_values(content, case_sensitive=True):
    """
    return all css color values
    
    >>> findall_color_values("#ff0000;")
    set(['ff0000'])
    
    >>> sorted(findall_color_values("#aabbcc; #112233;"))
    ['112233', 'aabbcc']
    
    # If case_sensitive==False: merge different large- and lowercase:
    >>> sorted(findall_color_values("#C9C573; #c9c573;"))
    ['C9C573', 'c9c573']
    >>> findall_color_values("#C9C573; #c9c573;", case_sensitive=False)
    set(['c9c573'])
    
    # Note: Didn't find 3 length color values! Use convert_3to6_colors()
    >>> findall_color_values("#abc;")
    set([])
    """
    colors = set(CSS_RE.findall(content))
    if not case_sensitive:
        colors = set([c.lower() for c in colors])
    return colors



def extract_colors(content, existing_color_dict=None):
    """   
    >>> extract_colors(".foo { color: #000000; }")
    ('.foo { color: {{ black }}; }', {'black': '000000'})
    
    it's case insensitivity:
    >>> extract_colors(".foo{color:#C9C573;} .bar{color:#c9c573;}")
    ('.foo{color:{{ darkkhaki_d20 }};} .bar{color:{{ darkkhaki_d20 }};}', {'darkkhaki_d20': 'c9c573'})
    
    Convert/merge 3 length values:
    >>> extract_colors(".foo{color:#11AAff;} .bar{color:#1af;}")
    ('.foo{color:{{ deepskyblue_d27 }};} .bar{color:{{ deepskyblue_d27 }};}', {'deepskyblue_d27': '11aaff'})
    
    You can give a existing color map:
    >>> extract_colors(".foo { color: #000000; }", existing_color_dict={"black":"000000"})
    ('.foo { color: {{ black }}; }', {'black': '000000'})
    
    Existing color names would not overwritten
    >>> extract_colors(".foo { color: #112233; }", existing_color_dict={"black":"000000"})
    ('.foo { color: {{ darkslategrey_d61 }}; }', {'black': '000000', 'darkslategrey_d61': '112233'})
    
    Existing color names would not overwritten
    >>> extract_colors(".foo { color: #010101; }", existing_color_dict={"black":"000000"})
    ('.foo { color: {{ black_2 }}; }', {'black_2': '010101', 'black': '000000'})
    
    Skip non color values:
    >>> extract_colors("to short #12345; to long #1234567; /* no # 123; color #aa11ff-value */")
    ('to short #12345; to long #1234567; /* no # 123; color #aa11ff-value */', {})
    """
    new_content = convert_3to6_colors(content)

    colors = findall_color_values(new_content)

    if existing_color_dict:
        color_dict = existing_color_dict.copy()
        exist_color = dict([(v, k) for k, v in existing_color_dict.iteritems()])
    else:
        color_dict = {}
        exist_color = {}

    for color in colors:
        color_lower = color.lower()

        if color_lower in exist_color:
            # color exist in other case
            color_name = exist_color[color_lower]
        else:# new color
            color_name = unique_color_name(color_dict, color)

            color_dict[color_name] = color_lower
            exist_color[color_lower] = color_name

        new_content = new_content.replace("#%s" % color, "{{ %s }}" % color_name)

    return new_content, color_dict


def replace_css_name(old_name, new_name, content):
    """
    >>> replace_css_name("old", "new", "color: {{ old }};")
    'color: {{ new }};'
    >>> replace_css_name("foo", "bar", "color: {{   foo   }};")
    'color: {{ bar }};'
    >>> replace_css_name("a", "b", "color: {{a}};")
    'color: {{ b }};'
    """
    return re.sub("\{\{\s*(" + old_name + ")\s*\}\}", "{{ %s }}" % new_name, content)


def get_new_css_names(existing_colors, content):
    """
    >>> get_new_css_names((), "{{ new }}")
    ['new']
    >>> get_new_css_names(("old",), "foo {{ old }} bar {{ new }}!")
    ['new']
    >>> get_new_css_names(("old",), "foo {{  old  }} bar {{  new  }}!")
    ['new']
    >>> get_new_css_names(("old",), "foo {{old}} bar {{new}}!")
    ['new']
    >>> get_new_css_names((), "foo {{ #ffddee }} bar {{new}}!")
    ['new']
    """
    # FIXME: Rexp should not match on colors!
    raw_colors = re.findall("\{\{\s*(.*?)\}\}", content)
    colors = set([color.strip() for color in raw_colors])
    new_colors = [color for color in colors if not color.startswith("#") and not color in existing_colors]
    return new_colors




if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."



# -*- coding: utf-8 -*-

"""
    PyLucid main_menu plugin
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    Generates a tree menu.

    :copyleft: 2005-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

from django.conf import settings
from django.contrib import messages

from pylucid_project.apps.pylucid.models import PageTree
from pylucid_project.apps.pylucid.decorators import render_to


def _debug(request, tree):
    messages.debug(request, "tree nodes: %r" % tree.nodes)
    messages.debug(request, "All PageTree:", PageTree.objects.all())
    messages.debug(request, "Current user:", request.user)
    messages.debug(request, "all accessible PageTree:", PageTree.objects.all_accessible(request.user).all())


@render_to("main_menu/main_menu.html")
def lucidTag(request, min=1, max=0):
    """
    You can split the menu with the optional min and max arguments:
    * min - The starting level (first level is 1)
    * max - The end level (without end, use 0)
    
    more info:
    http://www.pylucid.org/permalink/132/the-main-menu-plugin
    
    example:
        {% lucidTag main_menu %}
        {% lucidTag main_menu min=1 max=1 %}
        {% lucidTag main_menu min=2 max=0 %}
    """
    current_pagetree = request.PYLUCID.pagetree
    user = request.user

    # Get a pylucid.tree_model.TreeGenerator instance with all accessible PageTree for the current user
    tree = PageTree.objects.get_tree(user, filter_showlinks=True)

    # activate the current pagetree node (for main menu template)
    def get_first_showlink(pagetree):
        """ returns the first pagetree witch has showlinks==True, go recursive up to next parent """
        if pagetree.showlinks == False:
            if pagetree.parent == None:
                # not parent page available -> activate root node
                return None
            else:
                # go recursive up to next parent
                return get_first_showlink(pagetree.parent)
        return pagetree.id

    first_showlink_id = get_first_showlink(current_pagetree)
    try:
        tree.set_current_node(first_showlink_id)
    except KeyError, err:
        tree.set_current_node(None) # Root node
        if settings.DEBUG:
            messages.error(request, "Can't activate menu item %r KeyError: %s" % (current_pagetree, err))
            _debug(request, tree)

    tree.slice_menu(min, max)

    # add all PageMeta objects into tree
    tree.add_pagemeta(request)

#    _debug(request, tree)

    return {"nodes": tree.get_first_nodes()}














# coding: utf-8

from django import forms
from django.utils.translation import ugettext_lazy as _

from django_tools.fields.sign_separated import SignSeparatedFormField

from dbpreferences.forms import DBPreferencesBaseForm


class PyLucidCommentsPrefForm(DBPreferencesBaseForm):
    ban_limit = forms.IntegerField(
        help_text=_("Number of pause errors after IP would be banned."),
        initial=3, min_value=1, max_value=100
    )
    min_pause = forms.IntegerField(
        help_text=_("Minimum pause in seconds between two comments (Used 'REMOTE_ADDR' + username)"),
        initial=60, min_value=1, max_value=600
    )
    spam_keywords = SignSeparatedFormField(
        separator=",", strip_items=True, skip_empty=True,
        initial=[
            "www.", "://", "<", ">",
            "pr0n", "fuck", "blow", "pharmacy", "pills", "enlarge", "buy",
            "casino",
        ],
        help_text=_("Keywords for auto hide a new comment, for later moderation. (Comma seperated)"),
    )
    admins_notification = forms.BooleanField(
        initial=True,
        help_text=_("Email every settings.ADMINS after a new comment submitted."),
    )
    class Meta:
        app_label = 'pylucid_comments'



# coding: utf-8

"""
    PyLucid language plugin
    ~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2009-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.p

"""

__version__ = "$Rev:$"

from django.conf import settings
from django.contrib import messages
from django.core.exceptions import ValidationError
from django.http import HttpResponseRedirect

from django_tools.validators import validate_language_code

from pylucid_project.apps.pylucid.system import i18n
from pylucid_project.apps.pylucid.models import PageMeta, Language
from pylucid_project.apps.pylucid.decorators import render_to

from language.preference_forms import LanguagePrefForm


RESET_KEY = "reset"


def _can_reset():
    # Get preferences
    pref_form = LanguagePrefForm()
    pref_data = pref_form.get_preferences()
    return pref_data["add_reset_link"] or settings.DEBUG or settings.PYLUCID.I18N_DEBUG


@render_to("language/language_selector.html")
def lucidTag(request):
    """ insert language selector list into page """
    existing_languages = request.PYLUCID.languages

    if len(existing_languages) < 2:
        # Don't insert the language selector, if there only exist one language ;)
        if request.user.is_superuser:
            # Display a superuser a information (only one time per session)
            key = "useless_language_plugin"
            if key not in request.session:
                messages.info(request,
                    "It's useless to insert lucidtag language into this site!"
                    "For better performance you can remove this tag."
                )
                request.session[key] = True
        return

    current_language = request.PYLUCID.current_language
    current_pagetree = request.PYLUCID.pagetree
    absolute_url = current_pagetree.get_absolute_url()
    current_url = absolute_url.strip("/") # For {% url ... %}

    context = {
        "current_language": current_language,
        "current_url": current_url,
        "existing_languages": existing_languages,
        "add_reset_link": _can_reset(),
        "reset_key": RESET_KEY,
    }
    return context


def http_get_view(request):
    """
    Switch the client favored language and save it for every later requests.
    """
    user = request.user
    debug = settings.DEBUG or settings.PYLUCID.I18N_DEBUG

    raw_lang_code = request.GET.get("language", False)
    if not raw_lang_code:
        if settings.DEBUG or settings.PYLUCID.I18N_DEBUG:
            messages.error(request, "No language code!")
        return

    if raw_lang_code == RESET_KEY:
        # We should reset the current saved language data
        if not _can_reset():
            if debug:
                messages.error(request, "Error: i18n reset is off!")
            return
        return i18n.reset_language_settings(request)

    try:
        validate_language_code(raw_lang_code)
    except ValidationError, err:
        if debug:
            messages.error(request, "Wrong language code in url: %s" % err)
        return

    if raw_lang_code == request.PYLUCID.current_language.code:
        # Use the current lang entry and save it
        lang_entry = request.PYLUCID.current_language
        if debug:
            messages.info(request, "Save current lang entry.")
    else:
        existing_languages = Language.objects.all_accessible(user)
        try:
            lang_entry = existing_languages.get(code=raw_lang_code)
        except Language.DoesNotExist, err:
            if debug:
                messages.error(request, "Wrong lang code in get parameter: %s" % err)
            return

    i18n.activate_language(request, lang_entry, save=True)

    current_pagemeta = request.PYLUCID.pagemeta
    if current_pagemeta.language == lang_entry:
        if debug:
            messages.info(request, "Current page is in right language. No redirect needed.")
        return

    pagetree = request.PYLUCID.pagetree
    try:
        pagemeta = PageMeta.objects.get(pagetree=pagetree, language=lang_entry)
    except PageMeta.DoesNotExist, err:
        if debug:
            messages.info(request, "PageMeta doesn't exist in lang %r. So no redirect needed." % lang_entry)
        return

    # change only the lang code in the url:
    new_url = i18n.change_url(request, new_lang_code=pagemeta.language.code, save_get_parameter=False)

    # redirect, so the new selected language would be used
    return HttpResponseRedirect(new_url)



# coding:utf-8

from django import http
from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from pylucid_project.apps.pylucid.decorators import check_permissions, render_to
from pylucid_project.apps.pylucid.markup.converter import apply_markup
from pylucid_project.apps.pylucid.preference_forms import SystemPreferencesForm
from pylucid_project.apps.pylucid_admin.admin_menu import AdminMenu
from pylucid_project.utils.site_utils import get_site_preselection

from lexicon.forms import LexiconEntryForm
from lexicon.preference_forms import LexiconPrefForm



def install(request):
    """ insert PyLucid admin views into PageTree """
    output = []

    admin_menu = AdminMenu(request, output)
    menu_section_entry = admin_menu.get_or_create_section("create content")

    admin_menu.add_menu_entry(
        parent=menu_section_entry, url_name="Lexicon-new_entry",
        name="new lexicon entry", title="Create a new lexicon entry.",
    )

    return "\n".join(output)


@check_permissions(superuser_only=False, permissions=("lexicon.add_lexiconentry",))# "lexicon.add_links"))
@render_to("lexicon/new_entry.html")
def new_entry(request):
    """ create a new lexicon entry """
    context = {
        "title": _("Create a new lexicon entry"),
        "form_url": request.path,
    }

    if request.method == "POST":
        form = LexiconEntryForm(request.POST)
        if form.is_valid():
            if "preview" in request.POST:
                context["preview"] = apply_markup(
                    form.cleaned_data["content"], form.cleaned_data["markup"],
                    request.page_msg, escape_django_tags=True
                )
            else:
                instance = form.save()
                messages.info(request, _("Lexicon entry '%s' saved.") % instance.term)
                return http.HttpResponseRedirect(instance.get_absolute_url())
    else:
        # Get preferences
        pref_form = LexiconPrefForm()

        initial = {
            "sites": get_site_preselection(pref_form, request), # preselect sites field
            "language": request.PYLUCID.current_language.pk, # preselect current language
        }
        form = LexiconEntryForm(initial=initial)

    context["form"] = form
    return context




#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittests
    ~~~~~~~~~~~~~~~~~
    
    :copyleft: 2010 by the django-weave team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"

from django.conf import settings
from django.test.client import Client
from django.core.urlresolvers import reverse

from pylucid_project.tests.test_tools.basetest import BaseUnittest


class ShowInternalsTest(BaseUnittest):
    """
    inherited from BaseUnittest:
        - assertPyLucidPermissionDenied()
        - initial data fixtures with default test users
        - self.login()
    """
    def _pre_setup(self, *args, **kwargs):
        super(ShowInternalsTest, self)._pre_setup(*args, **kwargs)
        self.url = reverse("Internal-show_internals")

    def test_anonymous(self):
        response = self.client.get(self.url)
        self.assertRedirect(response, status_code=302,
            url="http://testserver/?auth=login&next_url=%s" % self.url
        )

    def test_summary(self):
        self.login("superuser")
        response = self.client.get(self.url)
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                '<title>PyLucid - Show internals</title>',
                'Cache status',
                'urlpatterns',
                'database backend information',
                "Internal-show_internals",
            ),
        )



if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
#    management.call_command('test', "pylucid_plugins.page_admin.tests.ConvertMarkupTest",
##        verbosity=0,
#        verbosity=1,
#        failfast=True
#    )
    management.call_command('test', __file__,
        verbosity=1,
#        verbosity=0,
#        failfast=True
    )



# coding: utf-8

from django.conf.urls.defaults import patterns, url

from update_journal import views

urlpatterns = patterns('',
    url(r'^(?P<filename>.+?)$', views.feed, name='UpdateJournal-feed'),
    url(r'^', views.select_feed, name='UpdateJournal-select_feed'),
)



#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittests
    ~~~~~~~~~~~~~~~~~
    
    Info:
        - PyLucid initial data contains english and german pages.
    
    :copyleft: 2010 by the django-weave team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"

from pylucid_project.tests.test_tools.basetest import BaseUnittest


class SearchTest(BaseUnittest):
    def test_get_search(self):
        response = self.client.get("/", data={"search": "PyLucid"})
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                '<title>PyLucid CMS - Advanced search</title>',
                '<input type="text" name="search" value="PyLucid" id="id_search" />',
                '<input type="submit" value="search" />',
                'Search Form',
                'Search results',
                "Search in ", " plugins",
                "duration: ", "sec.",
                "hits: ",
                '<strong>PyLucid</strong>',
            ),
            must_not_contain=(
                "Traceback", "Form errors", "field is required",
                "comments",
            )
        )

    def test_short_terms(self):
        response = self.client.get("/", data={"search": "py foo bar"})
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                '<title>PyLucid CMS - Advanced search</title>',
                "Ignore &#39;py&#39; (too small)",
                '<input type="text" name="search" value="py foo bar" id="id_search" />',
                '<input type="submit" value="search" />',
                'Search Form',
                'Search results',
                "Search in ", " plugins",
                "duration: ", "sec.",
                "hits: ",
            ),
            must_not_contain=("Traceback", "Form errors", "field is required")
        )

    def test_no_search_term_left(self):
        response = self.client.get("/", data={"search": "py xy z"})
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response,
            must_contain=(
                '<title>PyLucid CMS - Advanced search</title>',
                "Ignore &#39;py&#39; (too small)",
                "Ignore &#39;xy&#39; (too small)",
                "Ignore &#39;z&#39; (too small)",
                "Error: no search term left, can&#39;t search",
                '<input type="text" name="search" value="py xy z" id="id_search" />',
                '<input type="submit" value="search" />',
                'Search Form',
            ),
            must_not_contain=(
                "Traceback", "Form errors", "field is required"
                'Search results',
                "Search in ", " plugins",
                "duration: ", "sec.",
                "hits: ",
            )
        )



if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
#    management.call_command('test', "pylucid_plugins.page_admin.tests.ConvertMarkupTest",
##        verbosity=0,
#        verbosity=1,
#        failfast=True
#    )
    management.call_command('test', __file__,
        verbosity=1,
#        verbosity=0,
#        failfast=True
    )



# coding: utf-8

"""
    TODO:
    * handel proxy's 'HTTP_X_FORWARDED_FOR' values.
        See notes here:
        http://docs.djangoproject.com/en/1.0/ref/middleware/#reverse-proxy-middleware
"""

import datetime

from django.conf import settings
from django.contrib import messages
from django.contrib.admin.util import quote
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.managers import CurrentSiteManager
from django.contrib.sites.models import Site
from django.db import models
from django.utils.encoding import smart_unicode
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

# http://code.google.com/p/django-tools/
from django_tools.middlewares import ThreadLocal

# http://code.google.com/p/django-dbpreferences/
from dbpreferences.fields import DictField

from pylucid_project.apps.pylucid.models.base_models import UpdateInfoBaseModel


META_KEYS = (
    "REMOTE_ADDR", "REMOTE_USER", "REQUEST_METHOD", "QUERY_STRING",
    "HTTP_REFERER", "HTTP_USER_AGENT", "HTTP_ACCEPT_ENCODING", "HTTP_ACCEPT_LANGUAGE"
)

class LogEntryManager(models.Manager):


    def get_same_remote_addr(self, request):
        """
        return a QuerySet with all entries from the current remote Address.
        """
        current_remote_addr = request.META["REMOTE_ADDR"]
        queryset = self.model.on_site.all()
        queryset = queryset.filter(remote_addr=current_remote_addr)
        return queryset

    def last_remote_addr_actions(self, request, seconds):
        """
        creates a queryset with all items from the same REMOTE_ADDR in the last seconds.
        """
        queryset = self.get_same_remote_addr(request)

        timedelta = datetime.timedelta(seconds=seconds)
        point_in_time = datetime.datetime.now() - timedelta
        queryset = queryset.filter(createtime__gte=point_in_time)

        return queryset

    def request_limit(self, request, min_pause, ban_limit, app_label, action="error", no_page_msg=False):
        """
        Monitor request min_pause and ban_limit.
        """
        # Count the last requests for this app_label
        queryset = self.last_remote_addr_actions(request, min_pause)
        queryset = queryset.filter(app_label=app_label)
        queryset = queryset.filter(action=action)
        last_actions = queryset.count()

        if last_actions >= ban_limit:
            msg = _(
                "Add ban entry, because %(last_actions)s request for %(app_label)s"
                " in the last %(min_pause)ssec."
            ) % {
                "last_actions": last_actions,
                "app_label": app_label,
                "min_pause": min_pause,
            }
            self.log_action(app_label=app_label, action="ban ip", message=msg)

            from pylucid_project.apps.pylucid.models import BanEntry
            BanEntry.objects.add(request) # raise 404 after adding the client IP!

        if last_actions > 0:
            msg = _("Request too fast!")
            debug_msg = " (%s in the last %ssec. IP is blocked by %s overruns.)" % (
                last_actions, min_pause, ban_limit
            )
            if settings.DEBUG:
                msg += debug_msg
            if no_page_msg == False:
                messages.error(request, msg)
            LogEntry.objects.log_action(
                app_label=app_label, action="request aborted", message=debug_msg,
            )
            raise self.model.RequestTooFast(msg)

    def log_action(self, app_label, action, request=None, message=None, long_message=None, data=None):
        if request is None:
            request = ThreadLocal.get_current_request()

        kwargs = {
            "uri": request.build_absolute_uri(),
            "app_label": app_label,
            "action": action,
            "message": message,
            "long_message": long_message,
            "data": data,
        }

        if hasattr(request, "PYLUCID"):
            kwargs["used_language"] = request.PYLUCID.current_language

        for key in META_KEYS:
            value = request.META.get(key)
            if value and len(value) > 255:
                value = "%s..." % value[:252]
            kwargs[key.lower()] = value

        new_entry = self.model(**kwargs)
        new_entry.save()
        return new_entry


class LogEntry(UpdateInfoBaseModel):
    """
    PyLucid action log entries.

    inherited attributes from UpdateInfoBaseModel:
        createtime     -> datetime of creation
        lastupdatetime -> datetime of the last change
        createby       -> ForeignKey to user who creaded this entry
        lastupdateby   -> ForeignKey to user who has edited this entry
    """
    class RequestTooFast(Exception):
        pass

    objects = LogEntryManager()

    site = models.ForeignKey(Site, default=Site.objects.get_current)
    on_site = CurrentSiteManager()

    # Log information:
    app_label = models.CharField(_('App Label'), max_length=255, db_index=True,
        help_text="The App name witch has created this log entry."
    )
    action = models.CharField(_('Action'), max_length=128, db_index=True,
        help_text="Short action key. (e.g.: 'do search', 'login')"
    )
    message = models.CharField(_('Message'), max_length=255, blank=True, null=True,
        help_text="Short/one line log message. (e.g.: 'user FooBar login')"
    )
    long_message = models.TextField(_('long Message'), blank=True, null=True,
        help_text="Complete log message."
    )
    # From django-dbpreferences
    data = DictField(blank=True, null=True, help_text="serialized dictionary data")

    # Own meta data:
    uri = models.CharField(_('URL'), max_length=255,
        help_text="absolute URI form request.build_absolute_uri()"
    )
    used_language = models.ForeignKey("pylucid.Language", blank=True, null=True)

    # Data from request.META
    remote_addr = models.IPAddressField(_('Remote IP Address'), blank=True, null=True, db_index=True,
        help_text="The IP address of the client. From request.META['REMOTE_ADDR']"
    )
    remote_user = models.CharField(_('Remote User'), max_length=255, blank=True, null=True,
        help_text="The user authenticated by the web server, if any. From request.META['REMOTE_USER']"
    )
    request_method = models.CharField(_('Request Method'), max_length=8, blank=True, null=True,
        help_text="Request method, e.g.: 'GET', 'POST'. From request.META['REQUEST_METHOD']"
    )
    query_string = models.CharField(_('Query String'), max_length=255, blank=True, null=True,
        help_text="The query string, as a single **unparsed** string. From request.META['QUERY_STRING']"
    )
    http_referer = models.CharField(_('Referer'), max_length=255, blank=True, null=True,
        help_text="The referring page, if any. From request.META['HTTP_REFERER']"
    )
    http_user_agent = models.CharField(_('User Agent'), max_length=255, blank=True, null=True,
        help_text="The client's user-agent string. From request.META['HTTP_USER_AGENT']"
    )
    http_accept_encoding = models.CharField(_('Accept Encoding'), max_length=255, blank=True, null=True,
        help_text="from request.META['HTTP_ACCEPT_ENCODING']"
    )
    http_accept_language = models.CharField(_('Accept Language'), max_length=255, blank=True, null=True,
        help_text="from request.META['HTTP_ACCEPT_LANGUAGE']"
    )

    def get_absolute_url(self):
        return self.uri

    def __unicode__(self):
        return u"LogEntry %s %s %s" % (self.createby, self.createtime, self.action)

    class Meta:
        app_label = 'pylucid'
        verbose_name = _('log entry')
        verbose_name_plural = _('log entries')
        ordering = ('-createtime',)





# coding: utf-8

"""
    PyLucid admin url patterns
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author:$

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.conf.urls.defaults import patterns, url

# XXX: why doesn't this work: from system.admin_views import base_check, timezone
from admin_views import base_check, timezone

urlpatterns = patterns('',
    url(r'^base_check/$', base_check, name='System-base_check'),
    url(r'^timezone/$', timezone, name='System-timezone'),
)




#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittests
    ~~~~~~~~~~~~~~~~~
    
    Info:
        - PyLucid initial data contains english and german pages.
        - There exist only "PyLucid CMS" lexicon entry in english and german
    
    :copyleft: 2010 by the django-weave team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"

from django.conf import settings

from pylucid_project.tests.test_tools import basetest
from pylucid_project.apps.pylucid.models import PageContent

from lexicon.models import LexiconEntry


SUMMARY_URL = "/%s/lexicon/"
ENTRY_URL = "/%s/lexicon/detail/PyLucid%%20CMS/"


class LexiconPluginTestCase(basetest.BaseLanguageTestCase):
    """
    inherited from BaseUnittest:
        - initial data fixtures with default test users
        - self.login()
    
    inherited from BaseLanguageTest:
        - self.default_language - system default Language model instance (default: en instance)
        - self.other_lang_code - alternative language code than system default (default: 'de')
        - self.other_language - alternative Language mode instance (default: de instance)
        - assertContentLanguage() - Check if response is in right language
    """
    SUMMARY_MUST_CONTAIN_EN = (
        '<a href="/en/lexicon/detail/PyLucid%20CMS/">PyLucid CMS</a>',
        '<dd>PyLucid is the CMS thats built this page.</dd>',
    )
    SUMMARY_MUST_CONTAIN_DE = (
        '<a href="/de/lexicon/detail/PyLucid%20CMS/">PyLucid CMS</a>',
        '<dd>PyLucid ist ein flexibles, Open Source Webseiten Content Management System.</dd>',
    )
    ENTRY_MUST_CONTAIN_EN = (
        '<a href="/en/lexicon/detail/PyLucid CMS/" title="PyLucid CMS', # breadcrumbs
        '<dd>PyLucid CMS</dd>',
        '<dt>Short definition:</dt>',
        '<p>This pages are created by PyLucid ;)</p>',
        'Leave a comment</a>', # comments
    )
    ENTRY_MUST_CONTAIN_DE = (
        '<a href="/de/lexicon/detail/PyLucid CMS/" title="PyLucid CMS', # breadcrumbs
        '<dd>PyLucid CMS</dd>',
        '<dt>Kurzdefinition:</dt>',
        '<p>Diese Seiten werden mit PyLucid CMS generiert ;)</p>',
        'Leave a comment</a>', # comments
    )
    ENTRY_MUST_CONTAIN_ES = (
        '<a href="/es/lexicon/detail/Spanish/" title="Spanish: Spanish is a language ;)"', # breadcrumbs
        '<dd>Spanish</dd>',
        '<p>Spanish or Castilian (español or castellano) is a Romance language...</p>',
        'Leave a comment</a>', # comments
    )

    def assertLexiconPage(self, response, must_contain):
        self.failUnlessEqual(response.status_code, 200)
        self.assertResponse(response, must_contain=must_contain,
            must_not_contain=("Traceback", "XXX INVALID TEMPLATE STRING")
        )

    def assertSummaryEN(self, response):
        self.assertLexiconPage(response, must_contain=self.SUMMARY_MUST_CONTAIN_EN)

    def assertSummaryDE(self, response):
        self.assertLexiconPage(response, must_contain=self.SUMMARY_MUST_CONTAIN_DE)

    def assertEntryEN(self, response):
        self.assertLexiconPage(response, must_contain=self.ENTRY_MUST_CONTAIN_EN)

    def assertEntryDE(self, response):
        self.assertLexiconPage(response, must_contain=self.ENTRY_MUST_CONTAIN_DE)

    def assertEntryES(self, response):
        self.assertLexiconPage(response, must_contain=self.ENTRY_MUST_CONTAIN_ES)


class LexiconPluginTest1(LexiconPluginTestCase):
    """
    Tests with existing lexicon entries
    """
    def test_summary_en(self):
        response = self.client.get(
            SUMMARY_URL % self.default_language.code,
            HTTP_ACCEPT_LANGUAGE=self.default_language.code,
        )
        self.assertSummaryEN(response)
        self.assertContentLanguage(response, self.default_language)

    def test_summary_de(self):
        response = self.client.get(
            SUMMARY_URL % self.other_language.code,
            HTTP_ACCEPT_LANGUAGE=self.other_language.code,
        )
        self.assertSummaryDE(response)
        self.assertContentLanguage(response, self.other_language)

    def test_lexicon_entry_en(self):
        response = self.client.get(
            ENTRY_URL % self.default_language.code,
            HTTP_ACCEPT_LANGUAGE=self.default_language.code,
        )
        self.assertEntryEN(response)
        self.assertContentLanguage(response, self.default_language)

    def test_lexicon_entry_de(self):
        response = self.client.get(
            ENTRY_URL % self.other_language.code,
            HTTP_ACCEPT_LANGUAGE=self.other_language.code,
        )
        self.assertEntryDE(response)
        self.assertContentLanguage(response, self.other_language)

    def test_switch_url_language(self):
        """
        Request german language entry, but english ist current language
        -> redirect to english url
        """
        response = self.client.get(
            ENTRY_URL % self.other_language.code,
            HTTP_ACCEPT_LANGUAGE=self.default_language.code,
        )
        self.assertRedirect(
            response, url="http://testserver" + ENTRY_URL % self.default_language.code, status_code=302
        )

    def test_get_view(self):
        response = self.client.get("/?lexicon=PyLucid CMS")
        self.assertResponse(response,
            must_contain=(
                'PyLucid CMS',
                'lang="en"',
                '<a href="/en/lexicon/">&lt;&lt; goto lexicon summary page</a>',
                '<dt>Short definition:</dt>',
                '<p>This pages are created by PyLucid ;)</p>',
                'Leave a comment</a>',
            ),
            must_not_contain=("Traceback", "XXX INVALID TEMPLATE STRING")
        )



class LexiconPluginTest2(LexiconPluginTestCase, basetest.BaseMoreLanguagesTestCase):
    """
    Tests with some new lexicon entries
        
    inherited from BaseMoreLanguagesTestCase:
        - created languages: "es", "es-ar", "pt", "hr"
        - self.languages - A dict with language code as keys and language instance as values   
    """
    def _pre_setup(self, *args, **kwargs):
        """ create some blog articles """
        super(LexiconPluginTest2, self)._pre_setup(*args, **kwargs)

        defaults = {
            "markup": PageContent.MARKUP_CREOLE,
            "is_public": True,
        }

        self.entry_es = self.easy_create(LexiconEntry, defaults,
            term="Spanish",
            language=self.languages["es"],
            tags="shared, Spain, other",
            short_definition="Spanish is a language ;)",
            content="Spanish or Castilian (español or castellano) is a Romance language...",
        )

    def test_es_entry(self):
        response = self.client.get("/es/lexicon/detail/Spanish/", HTTP_ACCEPT_LANGUAGE="es")
        self.assertEntryES(response)

    def test_es_redirect(self):
        """
        the activated language is German, but we want a Spain lexicon entry -> Redirect to it 
        """
        response = self.client.get(
            "/es/lexicon/detail/Spanish/",
            HTTP_ACCEPT_LANGUAGE="not-exist;q=0.9,de;q=0.8,pt-br;q=0.7,es;q=0.5"
        )
        self.assertRedirect(
            response, url="http://testserver/es/lexicon/detail/Spanish/", status_code=302
        )


if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
#    management.call_command('test', "pylucid_plugins.lexicon.tests.LexiconPluginTest", verbosity=0)
    management.call_command('test', __file__,
        verbosity=1,
#        verbosity=0,
        failfast=True
    )



# coding: utf-8

"""
    PyLucid extrahead context middleware
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    replace <!-- ContextMiddleware extrahead --> in the global page template with
    all extra head html code, stored in request.PYLUCID.extrahead (pylucid.system.extrahead)
    
    Add all headfile links from pagetree.design.headfiles m2m.
    
    PyLucid plugins should use {% extrahead %} block tag (pylucid.defaulttags.extraheadBlock)
    in plugin template for insert e.g. CSS/JS file links into html head.

    Last commit info:
    ~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: JensDiemer $

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

__version__= "$Rev:$"

import os
import inspect

from django.conf import settings


class ContextMiddleware(object):
    """ replace <!-- ContextMiddleware extrahead --> in the global page template """
    def __init__(self, request, context):
        self.request = request
        self.extrahead = request.PYLUCID.extrahead # pylucid.system.extrahead
                            
    def _add_pagetree_headfiles(self):
        """ add all headfile links used in the current design. """
        pagetree = self.request.PYLUCID.pagetree
        design = pagetree.design
        
        colorscheme = design.colorscheme
        headfiles = design.headfiles.all()
        
        for headfile in headfiles:
            # Get a instance from pylucid_project.apps.pylucid.system.headfile.HeadfileLink():
            headfilelink = headfile.get_headfilelink(colorscheme)
            head_tag = headfilelink.get_head_tag()
            self.extrahead.append(head_tag)
        
    def render(self):
        """ return all extra head content with all headfiles from current used design """
        self._add_pagetree_headfiles()
        return "\n".join(self.extrahead)




# -*- coding: utf-8 -*-

"""
    PyLucid markup converter
    ~~~~~~~~~~~~~~~~~~~~~~~~

    apply a markup to a content

    Last commit info:
    ~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author$

    :copyleft: 2007-2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

import re


if __name__ == "__main__":
    # For doctest only
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "PyLucid.settings"


from django.conf import settings
from django.contrib import messages
from django.utils.safestring import mark_safe
from django.utils.encoding import smart_str, force_unicode

from django_tools.utils.messages import FileLikeMessages

from pylucid_project.utils.escape import escape_django_tags as escape_django_template_tags
from pylucid_project.utils.SimpleStringIO import SimpleStringIO
from pylucid_project.apps.pylucid.markup.django_tags import DjangoTagAssembler

from pylucid_project.apps.pylucid.models import PageContent

#______________________________________________________________________________
# MARKUP

BLOCK_RE = re.compile("\n{2,}")

LINK_RE = re.compile(
    r'''(?<!=")(?P<url>(http|ftp|svn|irc)://(?P<title>[^\s\<]+))(?uimx)'''
)

def fallback_markup(content):
    """
    A simplest markup, build only paragraphs.
    >>> fallback_markup("line one\\nline two\\n\\nnext block")
    '<p>line one<br />\\nline two</p>\\n\\n<p>next block</p>'

    >>> fallback_markup("url: http://pylucid.org END")
    '<p>url: <a href="http://pylucid.org">pylucid.org</a> END</p>'
    """
    content = content.replace("\r\n", "\n").replace("\r", "\n")
    blocks = BLOCK_RE.split(content)
    blocks = [line.replace("\n", "<br />\n") for line in blocks]
    content = "<p>" + "</p>\n\n<p>".join(blocks) + "</p>"
    content = LINK_RE.sub(r'<a href="\g<url>">\g<title></a>', content)
    return content

def apply_tinytextile(content, page_msg):
    """ tinyTextile markup """
    from pylucid_project.apps.pylucid.markup.tinyTextile import TinyTextileParser
    out_obj = SimpleStringIO()
    markup_parser = TinyTextileParser(out_obj, page_msg)
    markup_parser.parse(smart_str(content))
    return force_unicode(out_obj.getvalue())


def apply_textile(content, page_msg):
    """ Original textile markup """
    try:
        import textile
    except ImportError:
        page_msg(
            "Markup error: The Python textile library isn't installed."
            " Download: http://cheeseshop.python.org/pypi/textile"
        )
        return fallback_markup(content)
    else:
        return force_unicode(textile.textile(
            smart_str(content),
            encoding=settings.DEFAULT_CHARSET,
            output=settings.DEFAULT_CHARSET
        ))

def apply_markdown(content, page_msg):
    """ Markdown markup """
    try:
        import markdown
    except ImportError:
        page_msg(
            "Markup error: The Python markdown library isn't installed."
            " Download: http://sourceforge.net/projects/python-markdown/"
        )
        return fallback_markup(content)
    else:
        # unicode support only in markdown v1.7 or above.
        # version_info exist only in markdown v1.6.2rc-2 or above.
        if getattr(markdown, "version_info", None) < (1, 7):
            return force_unicode(markdown.markdown(smart_str(content)))
        else:
            return markdown.markdown(content)


def apply_restructuretext(content, page_msg):
    try:
        from docutils.core import publish_parts
    except ImportError:
        page_msg(
            "Markup error: The Python docutils library isn't installed."
            " Download: http://docutils.sourceforge.net/"
        )
        return fallback_markup(content)
    else:
        docutils_settings = getattr(
            settings, "RESTRUCTUREDTEXT_FILTER_SETTINGS", {}
        )
        parts = publish_parts(
            source=content, writer_name="html4css1",
            settings_overrides=docutils_settings
        )
        return parts["fragment"]



def apply_creole(content):
    """
    Use python-creole:
    http://code.google.com/p/python-creole/

    We used verbose=1 for inser error information (e.g. not existing macro)
    into the generated page
    """
    from pylucid_project.apps.pylucid.markup import PyLucid_creole_macros

    from creole import Parser
    from creole.creole2html import HtmlEmitter

    # Create document tree from creole markup
    document = Parser(content).parse()

    # Build html code from document tree
    return HtmlEmitter(document, macros=PyLucid_creole_macros, verbose=0).emit()


def convert(raw_content, markup_no, page_msg):
    if markup_no == PageContent.MARKUP_TINYTEXTILE: # PyLucid's TinyTextile
        html_content = apply_tinytextile(raw_content, page_msg)
    elif markup_no == PageContent.MARKUP_TEXTILE: # Textile (original)
        html_content = apply_textile(raw_content, page_msg)
    elif markup_no == PageContent.MARKUP_MARKDOWN:
        html_content = apply_markdown(raw_content, page_msg)
    elif markup_no == PageContent.MARKUP_REST:
        html_content = apply_restructuretext(raw_content, page_msg)
    elif markup_no == PageContent.MARKUP_CREOLE:
        html_content = apply_creole(raw_content)
    elif markup_no in (PageContent.MARKUP_HTML, PageContent.MARKUP_HTML_EDITOR):
        html_content = raw_content
    else:
        raise AssertionError("markup no %r unknown!" % markup_no)

    return html_content


def apply_markup(raw_content, markup_no, request, escape_django_tags=False):
    """ render markup content to html. """
    page_msg = FileLikeMessages(request, messages.INFO)

    assemble_tags = markup_no not in (PageContent.MARKUP_HTML, PageContent.MARKUP_HTML_EDITOR)
    if assemble_tags:
        # cut out every Django tags from content
        assembler = DjangoTagAssembler()
        raw_content2, cut_data = assembler.cut_out(raw_content)
    else:
        raw_content2 = raw_content

    html_content = convert(raw_content2, markup_no, page_msg)

    if assemble_tags:
        # reassembly cut out django tags into text
        if not isinstance(html_content, unicode):
            if settings.DEBUG:
                markup_name = PageContent.MARKUP_DICT[markup_no]
                page_msg("Info: Markup converter %r doesn't return unicode!" % markup_name)
            html_content = force_unicode(html_content)

        html_content2 = assembler.reassembly(html_content, cut_data)
    else:
        # html "markup" used
        html_content2 = raw_content

    if escape_django_tags:
        html_content2 = escape_django_template_tags(html_content2)

    return mark_safe(html_content2) # turn Django auto-escaping off


def convert_markup(raw_content, source_markup_no, dest_markup_no, request):
    """
    Convert one markup in a other.
    """
    page_msg = FileLikeMessages(request, messages.INFO)

    html_source = source_markup_no in (PageContent.MARKUP_HTML, PageContent.MARKUP_HTML_EDITOR)
    html_dest = dest_markup_no in (PageContent.MARKUP_HTML, PageContent.MARKUP_HTML_EDITOR)

    if source_markup_no == dest_markup_no or (html_source and html_dest):
        # Nothing to do ;)
        return raw_content

    if not html_dest and dest_markup_no != PageContent.MARKUP_CREOLE:
        raise NotImplementedError("Converting into %r not supported." % dest_markup_no)

    if html_source: # Source markup is HTML
        html_content = raw_content
    else:
        # cut out every Django tags from content
        assembler = DjangoTagAssembler()
        raw_content2, cut_data = assembler.cut_out(raw_content)

        # convert to html
        html_content = convert(raw_content2, source_markup_no, page_msg)

    if html_dest: # Destination markup is HTML
        new_content = html_content
    else:
        # Skip: if dest_markup_no == PageContent.MARKUP_CREOLE: - only creole supported here
        from creole import html2creole
        new_content = html2creole(html_content)

    if not html_source: # Source markup is not HTML
        # reassembly cut out django tags into text
        new_content = assembler.reassembly(new_content, cut_data)

    return new_content



if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."



# coding: utf-8

"""
    Convert PageContent markup.
"""


from django import http
from django.conf import settings
from django.contrib import messages
from django.db import transaction
from django.utils.translation import ugettext as _

from pylucid_project.apps.pylucid.decorators import check_permissions, render_to
from pylucid_project.apps.pylucid.markup import converter
from pylucid_project.apps.pylucid.markup import hightlighter
from pylucid_project.apps.pylucid.models import PageContent

from page_admin.forms import ConvertMarkupForm


@check_permissions(superuser_only=False, permissions=("pylucid.change_pagecontent",))
@render_to("page_admin/convert_markup.html")
def convert_markup(request, pagecontent_id=None):
    """
    convert a PageContent markup
    """
    if not pagecontent_id:
        raise

    def _error(pagecontent_id, err):
        err_msg = _("Wrong PageContent ID.")
        if settings.DEBUG:
            err_msg += " (ID: %r, original error was: %r)" % (pagecontent_id, err)
        messages.error(request, err_msg)

    try:
        pagecontent_id = int(pagecontent_id)
    except Exception, err:
        return _error(pagecontent_id, err)

    try:
        pagecontent = PageContent.objects.get(id=pagecontent_id)
    except PageContent.DoesNotExist, err:
        return _error(pagecontent_id, err)

    absolute_url = pagecontent.get_absolute_url()
    context = {
        "title": _("Convert '%s' markup") % pagecontent.get_name(),
        "form_url": request.path,
        "abort_url": absolute_url,
        "current_markup": pagecontent.MARKUP_DICT[pagecontent.markup],
        "pagecontent": pagecontent,
    }

    if request.method != "POST":
        form = ConvertMarkupForm(instance=pagecontent)
    else:
        form = ConvertMarkupForm(request.POST, instance=pagecontent)
        #messages.info(request, request.POST)
        if form.is_valid():
            cleaned_data = form.cleaned_data
            dest_markup_no = int(cleaned_data["dest_markup"])
            original_markup = cleaned_data["content"]
            try:
                new_markup = converter.convert_markup(
                    original_markup,
                    source_markup_no=pagecontent.markup,
                    dest_markup_no=dest_markup_no,
                    request=request
                )
            except Exception, err:
                messages.error(request, "Convert error: %s" % err)
            else:
                if "preview" not in request.POST:
                    # Save converted markup and redirect to the updated page
                    sid = transaction.savepoint()
                    try:
                        pagecontent.content = new_markup
                        pagecontent.markup = dest_markup_no
                        pagecontent.save()
                    except:
                        transaction.savepoint_rollback(sid)
                        raise
                    else:
                        transaction.savepoint_commit(sid)
                        messages.info(request, _("Content page %r updated.") % pagecontent)
                        return http.HttpResponseRedirect(pagecontent.get_absolute_url())

                # preview markup convert:

                context["new_markup"] = new_markup

                converted_html = converter.apply_markup(
                    new_markup, dest_markup_no, request, escape_django_tags=True
                )
                context["converted_html"] = converted_html

                if cleaned_data["verbose"]:
                    context["original_markup"] = original_markup

                    orig_html = converter.apply_markup(
                        original_markup, pagecontent.markup, request, escape_django_tags=True
                    )
                    context["orig_html"] = orig_html

                    def strip_whitespace(html):
                        return "\n".join([line.strip() for line in html.splitlines() if line.strip()])

                    # Remove whitespace from html code.
                    orig_html2 = strip_whitespace(orig_html)
                    converted_html2 = strip_whitespace(converted_html)

                    if orig_html2 == converted_html2:
                        context["diff_is_the_same"] = True
                    else:
                        context["pygmentize_diff"] = hightlighter.get_pygmentize_diff(orig_html2, converted_html2)

                        # get the EditableHtmlHeadFile path to pygments.css (page_msg created, if not exists)
                        pygments_css_path = hightlighter.get_pygments_css(request)
                        context["pygments_css"] = pygments_css_path

    context.update({
        "form": form,
    })
    return context



# -*- coding: utf-8 -*-

"""
    PyLucid breadcrumb plugin
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    Generates a horizontal backlink bar.

    Last commit info:
    ~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: JensDiemer $

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

__version__ = "$Rev:$"

from django.template import RequestContext
from django.shortcuts import render_to_response

from pylucid_project.apps.pylucid.models import PageTree

from breadcrumb.preference_forms import BreadcumbPrefForm


class ContextMiddleware(object):
    def __init__(self, request, context):
        self.request = request
        self.context = context

        # Get preferences
        pref_form = BreadcumbPrefForm()
        self.pref_data = pref_form.get_preferences()

        # Get all pages back to the root page as a list
        self.linklist = PageTree.objects.get_backlist(request)

    def add_link(self, name, title="", url=None):
        """
        Can be called from plugins, to insert own virtual sub pages.
        if url==None: use full current request path
        """
        url = url or self.request.get_full_path()
        self.linklist.append({"name": name, "title": title, "url": url})

    def render(self):
        context = {
            "preferences": self.pref_data,
            "linklist": self.linklist,
        }
        return render_to_response('breadcrumb/breadcrumb.html', context,
            context_instance=RequestContext(self.request)
        )



# coding:utf-8

"""
    search all PageContent pages
"""

from django.contrib.sites.models import Site

from tagging.utils import parse_tag_input

from blog.models import BlogEntry


def get_search_results(request, search_languages, search_strings, search_results):
    queryset = BlogEntry.on_site

    # Only public blog items:
    queryset = queryset.filter(is_public=True)

    # Only items in the selected search language
    queryset = queryset.filter(language__in=search_languages)

    for term in search_strings:
        queryset = queryset.filter(content__icontains=term)

    for item in queryset:
        meta_content = item.tags
        #print "meta: %r" % meta_content

        search_results.add(
            model_instance=item,

            # displayed headline of the result hit
            headline=item.headline,

            # displayed in the result list
            language=item.language,

            # Link to the hit
            url=item.get_absolute_url(),

            # the main content -> result lines would be cut out from hits in this content
            content=item.content,

            # hits in meta content has a higher score, but the content would not displayed 
            meta_content=meta_content,
        )



# coding:utf-8

"""
    PyLucid
    ~~~~~~~

    :copyleft: 2009-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

from django import http
from django.conf import settings
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User, Group
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

from pylucid_project.apps.pylucid.decorators import check_permissions, render_to
from pylucid_project.apps.pylucid.models import PageTree, PageMeta, PageContent, PluginPage, Design
from pylucid_project.apps.pylucid.preference_forms import SystemPreferencesForm
from pylucid_project.apps.pylucid.system import pylucid_plugin, pylucid_objects
from pylucid_project.apps.pylucid_admin.models import PyLucidAdminPage
from pylucid_project.system.pylucid_plugins import PYLUCID_PLUGINS



@login_required
@render_to("pylucid_admin/menu.html")
def menu(request):
    return {"title": "PyLucid admin menu"}


@check_permissions(superuser_only=True)
@render_to("pylucid_admin/install.html")
def install_pylucid(request):
    """
    FIXME: obsolete???
    """
    output = []
    output.append("*** PyLucid install:")

    sys_pref_form = SystemPreferencesForm()

    # ------------------------------------------------------------------------
    pylucid_admin_group, created = Group.objects.get_or_create(name=settings.ADMIN.USER_GROUP)
    if created:
        output.append("User group '%s' created." % settings.ADMIN.USER_GROUP)
    else:
        output.append("User group '%s' exist." % settings.ADMIN.USER_GROUP)

    # ------------------------------------------------------------------------
    pylucid_admin_design, created = Design.objects.get_or_create(
        name=settings.ADMIN.DESIGN_NAME, defaults={"template": "pylucid_admin/menu.html"}
    )
    # TODO: Add to all sites?
    #design.site.add(site)
    if created:
        output.append("Design '%s' created." % settings.ADMIN.DESIGN_NAME)
    else:
        output.append("Design '%s' exist." % settings.ADMIN.DESIGN_NAME)

    # Add the Design id into the preferences
    sys_pref_form["pylucid_admin_design"] = pylucid_admin_design.id
#
#    # ------------------------------------------------------------------------    
#    pylucid_admin_pagetree, created = PageTree.objects.get_or_create(
#        slug="PyLucidAdmin", parent=None, site=Site.objects.get_current(),
#        defaults={
#            "design": pylucid_admin_design,
#            "type": PageTree.PAGE_TYPE,
#        }
#    )
#    pylucid_admin_pagetree.permitViewGroup = pylucid_admin_group
#
#    url = pylucid_admin_pagetree.get_absolute_url()
#    if created:
#        #tree_entry.save()
#        output.append("PageTree '%s' created." % url)
#    else:
#        output.append("PageTree '%s' exist." % url)
#
#    # Add the PageTree id into the preferences
#    sys_pref_form["pylucid_admin_pagetree"] = pylucid_admin_pagetree.id
#
#    # ------------------------------------------------------------------------
#    pylucid_admin_pagemeta, created = PageMeta.objects.get_or_create(
#        page=pylucid_admin_pagetree,
#        language=request.PYLUCID.current_language, # FIXME: Create in all existing languages?
#        defaults={"name": "PyLucid Admin", "robots":"noindex,nofollow"}
#    )
#    if created:
#        output.append("PageMeta %r created." % pylucid_admin_pagemeta)
#    else:
#        output.append("PageMeta %r exist." % pylucid_admin_pagemeta)
#
#    # ------------------------------------------------------------------------
#    pylucid_admin_pagecontent, created = PageContent.objects.get_or_create(
#        page=pylucid_admin_pagetree,
#        language=request.PYLUCID.current_language, # FIXME: Create in all existing languages?
#        pagemeta=pylucid_admin_pagemeta,
#        defaults={
#            "content":"PyLucid Admin section. Please seletect a menu item.",
#            "markup": PageContent.MARKUP_CREOLE,
#        }
#    )
#    if created:
#        output.append("PageContent %r created." % pylucid_admin_pagecontent)
#    else:
#        output.append("PageContent %r exist." % pylucid_admin_pagecontent)

    # Save new preferences
    sys_pref_form.save()
    output.append("System preferences saved.")

    context = {
        "title": "PyLucid - install",
        "output": output,
    }
    return context


@check_permissions(superuser_only=True)
@render_to("pylucid_admin/install.html")
def install_plugins(request):
    """
    Simple call all plugin install view, if exist.
    
    TODO: create a "install plugins" managment command and call it here! (This is useful for unittests)
    """
    output = []

    # Delete all items
    PyLucidAdminPage.objects.all().delete()

    output.append("*** Install Plugins:")

    filename = settings.ADMIN.VIEW_FILENAME
    view_name = settings.ADMIN.PLUGIN_INSTALL_VIEW_NAME

    for plugin_name, plugin_instance in PYLUCID_PLUGINS.iteritems():
        try:
            response = plugin_instance.call_plugin_view(request, filename, view_name, method_kwargs={})
        except Exception, err:
            if str(err).endswith("No module named %s" % filename):
                # Plugin has no install API
                if settings.DEBUG:
                    output.append("Skip plugin %r, because it has no install view (%s)" % (plugin_name, err))
                continue

            messages.error(request, "failed call %s.%s" % (plugin_name, view_name))
            continue

        output.append("_" * 79)
        output.append(" *** install plugin %r ***" % plugin_name)
        assert isinstance(response, basestring) == True, "Plugin install view must return a basestring!"
        output.append(response)

        output.append(" --- %s END ---" % plugin_name)
        output.append("")

    context = {
        "title": "PyLucid - Plugin install",
        "output": output,
    }
    return context



# coding: utf-8

import traceback

from django.conf import settings
from django.contrib.redirects.models import Redirect
from django.http import Http404
from django.utils.encoding import smart_str

from pylucid_project.apps.pylucid.models import LogEntry
from pylucid_project.apps.pylucid.system import pylucid_objects
from pylucid_project.apps.pylucid.preference_forms import SystemPreferencesForm



class PyLucidMiddleware(object):
    def process_request(self, request):
        """ Add PyLucid objects to the request object """
        request.PYLUCID = pylucid_objects.PyLucidRequestObjects(request)

    def process_exception(self, request, exception):
        if isinstance(exception, Http404): # Handle 404 page not found errors

            sys_pref_form = SystemPreferencesForm()
            sys_pref = sys_pref_form.get_preferences()
            log404_verbosity = sys_pref.get("log404_verbosity", sys_pref_form.LOG404_NOREDIRECT)

            if log404_verbosity == sys_pref_form.LOG404_NOTHING:
                # Don't log 'Page not found' errors.
                return

            # Check if there exist a django.contrib.redirects entry for this url
            path = request.get_full_path()
            try:
                r = Redirect.objects.get(site__id__exact=settings.SITE_ID, old_path=path)
            except Redirect.DoesNotExist:
                pass
            else:
                # Redirect entry exist
                if log404_verbosity == sys_pref_form.LOG404_NOREDIRECT:
                    # Log only 'Page not found' if no redirect for the url exists.
                    return

                LogEntry.objects.log_action(
                    app_label="pylucid", action="PyLucidMiddleware.process_exception()",
                    message="Redirect for %r exist." % path
                )
                return

        # cut exception message text to LogEntry.message max_length, to get no "Data truncated..." warning
        message = smart_str(exception, errors='replace')[:255]

        LogEntry.objects.log_action(
            app_label="pylucid", action="PyLucidMiddleware.process_exception()", message=message,
            long_message=traceback.format_exc()
        )



# coding:utf-8

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.models import User, Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.core import urlresolvers
from django.core.cache import cache
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import ugettext as _

from django_tools.middlewares import ThreadLocal

from pylucid_project.apps.pylucid.tree_model import BaseTreeModel, TreeManager, TreeGenerator
from pylucid_project.apps.pylucid.models.base_models import UpdateInfoBaseModel
from django_tools.utils.messages import failsafe_message


class PyLucidAdminManager(TreeManager):
    def get_tree_for_user(self, user):
        filtered_items = self.get_for_user(user)
        tree = TreeGenerator(filtered_items)        
        #tree.debug()

        # Hide categories, if they has sub entry links        
        for node in tree.iter_flat_list():
            if not node.data["absolute_url"] and not node.subnodes:
                node.visible=False
        
        return tree

    def get_for_user(self, user):
        """
        returns only the menu items, for which the user has the rights.
        """
        all_items = self.all().order_by("position", "name")
        filtered_items = []
        for item in all_items:
            access_permissions = item.get_permissions()
            superuser_only, permissions, must_staff = access_permissions
                      
            if superuser_only == True and user.is_superuser != True:
                continue
            if must_staff == True and user.is_staff != True:
                continue         
            if not user.has_perms(permissions):
                continue

            filtered_items.append({
                "id": item.id,
                "parent": getattr(item.parent, "id", None),
                "absolute_url": item.get_absolute_url(),
                "name": item.name,
                "title": item.title,
            })

        return filtered_items

class PyLucidAdminPage(BaseTreeModel, UpdateInfoBaseModel):
    """
    PyLucid Admin page tree
    
    inherited attributes from TreeBaseModel:
        parent
        position

    inherited attributes from UpdateInfoBaseModel:
        createtime     -> datetime of creation
        lastupdatetime -> datetime of the last change
        createby       -> ForeignKey to user who creaded this entry
        lastupdateby   -> ForeignKey to user who has edited this entry
    """
    objects = PyLucidAdminManager()

    #TODO: check if url_name is unique. We can't set unique==True,
    #      because menu section has always url_name=None
    url_name = models.CharField(blank=True, null=True, max_length=256,
        help_text="Name of url, defined in plugin/admin_urls.py"
    )

    name = models.CharField(max_length=150, unique=True,
        help_text="Sort page name (for link text in e.g. menu)"
    )
    title = models.CharField(blank=True, null=False, max_length=256,
        help_text="A long page title (for e.g. page title or link title text)"
    )

    get_pagetree = models.BooleanField(default=False,
        verbose_name="get PageTree",
        help_text="Add current PageTree ID via GET Parameter to the url, if available"
    )
    get_pagemeta = models.BooleanField(default=False,
        verbose_name="get PageMeta",
        help_text="Add current PageMeta ID via GET Parameter to the url, if available"
    )
    get_page = models.BooleanField(default=False,
        verbose_name="get PageContent/PluginPage",
        help_text="Add current PageContent or current PluginPage ID via GET Parameter to the url, if available"
    )

    def clean_fields(self, exclude):
        # check if parent is the same entry: child <-> parent loop:
        super(PyLucidAdminPage, self).clean_fields(exclude)

        # check if url_name is unique. We can't set unique==True,
        # because menu section has always url_name=None
        if "url_name" not in exclude and self.url_name is not None:
            queryset = PyLucidAdminPage.objects.filter(url_name=self.url_name)

            # Exclude the current object from the query if we are editing an
            # instance (as opposed to creating a new one)
            if self.pk is not None:
                queryset = queryset.exclude(pk=self.pk)

            exists = queryset.count()
            if exists:
                msg = "url name %r exist!" % self.url_name
                message_dict = {"url_name": (msg,)}
                raise ValidationError(message_dict)

    def get_url(self):
        """
        reverse the url name.
        
        Create error message if NoReverseMatch and return None.
        e.g.: Plugin deleted in filesystem, but still exist in database.
        
        FIXME: Can we get it faster and not with resolve the url?
        """
        try:
            url = urlresolvers.reverse(self.url_name)
        except urlresolvers.NoReverseMatch, err:
            msg = (
                "Can't resolve url %r for plugin %r: %s"
                " - (To fix this: run 'install plugins' again.)"
            ) % (self.url_name, self.name, err)
            failsafe_message(msg)
        else:
            return url

    def get_permissions(self):
        """
        returns the access permissions for this menu entry.
        TODO: Should be cache this?
        """
        if not self.url_name: # a menu section
            # TODO: Check if at least one sub entry is accessible.
            return (False, (), False)

        url = self.get_url()
        if url is None: # can't resolve url, message was created.
            return (True, (), True) # view can only superusers use

        # Get the view function for this url_name
        view_func, func_args, func_kwargs = urlresolvers.resolve(url)

        # get the rights from pylucid_project.apps.pylucid.decorators.check_permissions
        try:
            access_permissions = view_func.access_permissions
        except AttributeError, err:
            # If no permissions available, fallback to superuser only
            request = ThreadLocal.get_current_request()
            if settings.DEBUG or request.user.is_staff:
                messages.error(request, (
                    "The view %s for url %r has no permissions attribute!"
                    " Please use pylucid_project.apps.pylucid.decorators.check_permissions!"
                    ) % (view_func.__name__, self.url_name)
                )
            access_permissions = (True, (), True)

        return access_permissions

#    def save(self, *args, **kwargs):
#        """
#        After change, deletes panel_extra from cache.
#        cache filled in pylucid_plugins.admin_menu.views.panel_extras()
#        """
#        for user_id in User.objects.values_list('id', flat=True):
#            cache_key = "panel_extras_%s" % user_id
#            cache.delete(cache_key)
#
#        return super(PyLucidAdminPage, self).save(*args, **kwargs)

    def __unicode__(self):
        return u"PyLucidAdminPage %r (%r)" % (self.name, self.get_absolute_url())

    def get_absolute_url(self):
        """
        absolute url (without domain/host part)
        TODO: Should be used a cache here?
        """
        if not self.url_name:
            return "" # menu section

        url = self.get_url()
        if url is None: # can't resolve url, message was created.
            return "#resolve-error" # XXX: return something else?

        request = ThreadLocal.get_current_request()
        get_data = {}
        if self.get_pagetree and hasattr(request.PYLUCID, "pagetree"):
            get_data["pagetree"] = request.PYLUCID.pagetree.pk
        if self.get_pagemeta and hasattr(request.PYLUCID, "pagemeta"):
            get_data["pagemeta"] = request.PYLUCID.pagemeta.pk
        if self.get_page:
            if hasattr(request.PYLUCID, "pagecontent"):
                get_data["pagecontent"] = request.PYLUCID.pagecontent.pk
            elif hasattr(request.PYLUCID, "pluginpage"):
                get_data["pluginpage"] = request.PYLUCID.pluginpage.pk

        if get_data:
            # FIXME: There must be a better was to to this.
            # TODO: escape it.
            url += "?" + "&".join(["%s=%s" % (key, value) for key, value in get_data.items()])

        return url

    class Meta:
        verbose_name = _('PyLucid admin page')
        verbose_name_plural = _('PyLucid admin pages')
        ordering = ("name", "url_name",)




from django import template

from pylucid_project.apps.pylucid.models import Page08

register = template.Library()

@register.inclusion_tag('pylucid/simple_menu.html')
def simple_menu():
    pages = Page08.objects.all()
    context = {
        "pages": pages,
    }
    return context

#from django.template import add_to_builtins
#add_to_builtins('PyLucid.template_addons')


# coding: utf-8

"""
    PyLucid lexicon plugin
    ~~~~~~~~~~~~~~~~~~~~~~

    Related threads in german python forum:
        http://www.python-forum.de/topic-19964.html
        http://www.python-forum.de/topic-22102.html    
   
    Last commit info:
    ~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: JensDiemer $

    :copyleft: 2009-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

import os
import re
import HTMLParser
from xml.sax.saxutils import quoteattr


if __name__ == "__main__":
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"
    virtualenv_file = "../../../../../bin/activate_this.py"
    execfile(virtualenv_file, dict(__file__=virtualenv_file))



class NoneHTMLParser(object, HTMLParser.HTMLParser):
    """
    Parse the html code with HTMLParser and rebuilt it in self.html
    FIXME: Changes from original html to regenerated:
        - entityref lile umlaute would be replace with the real character, e.g.: &auml; -> ä
        - spaces can be changed, e.g:
            old: <a  href="foo"  >bar</a>
            new: <a href="foo">bar</a>
            old: <br/>
            new: <br />
        - empty attributes are made xhtml conform, e.g:
            old: <td nowrap>
            new: <td nowarp="nowarp">
        - add quote sign to attributes without quote sign, e.g:
            old: <table border=1>
            new: <table border="1">
        - newlines in tags are removes:
            old: <link\nrel="...
            new: <link rel="...
        - attribute quoting can be changed:
            old: title="foo &quot;bar&quot;"
            new: title='foo "bar"'
    """
    def __init__(self):
        # Note: HTMLPaser is a oldstyle class!
        self.reset() # Initialize and reset this HTMLParser instance.

        self.html = ""

    def _add_attrs(self, attrs):
        if attrs:
            attr_list = []
            for attr, value in attrs:
                if value is None:
                    # convert empty attrs to xhtml conform attributes
                    # e.g.: <td nowrap>  ->>  <td nowarp="nowarp">
                    value = attr
                # FIXME: original quoting can be changed
                # e.g: title="foo &quot;bar&quot;" -> title='foo "bar"'
                value = quoteattr(value)
                attr_list.append('%s=%s' % (attr, value))

            self.html += " " + " ".join(attr_list)

    def handle_startendtag(self, tag, attrs):
#        print "handle start+end tag: %r attrs: %r" % (tag, attrs)
        self.html += "<" + tag
        self._add_attrs(attrs)
        self.html += " />"

    def handle_starttag(self, tag, attrs):
#        print "handle start tag: %r attrs: %r" % (tag, attrs)
        self.html += "<" + tag
        self._add_attrs(attrs)
        self.html += ">"

    def handle_endtag(self, tag):
#        print "handle end tag: %r" % tag
        self.html += "</%s>" % tag

    def handle_charref(self, name):
#        print "handle character reference: %r" % name
        self.html += "&#%s;" % name

    def handle_entityref(self, name):
#        print "handle entity reference: %r" % name
        self.html += "&%s;" % name

    def handle_data(self, data):
#        print "handle data: %r" % data
        self.html += data

    def handle_comment(self, data):
#        print "handle comment: %r" % data
        self.html += "<!--%s-->" % data

    def handle_decl(self, decl):
#        print "handle declaration: %r" % decl
        self.html += "<!%s>" % decl

    def handle_pi(self, data):
        print "handle processing instruction:", data

    def unknown_decl(self, data):
        self.error("unknown declaration: %r" % (data,))



class SubHtml(NoneHTMLParser):
    """
    replace all lexicon words in handle_data()
    """
    def __init__(self, lexicon_data, skip_tags):
        super(SubHtml, self).__init__()

        # forming a dict from the skip_tags list, for faster lookup
        self.skip_tags = dict([(tag.lower(), None) for tag in skip_tags])

        self.lexicon_data = lexicon_data

        self.regex = self._build_regex() # Build the regexp to find all lexicon words

        self.in_skip_tag = None # Storage if we are in a skip_tags

    def _build_regex(self):
        keys = self.lexicon_data.keys()

        # Sort longest to shortest
        keys.sort(cmp=lambda x, y: cmp(len(y), len(x)))

        # match on all existing keys with ignorecase
        regex = re.compile(
            "(?<=[\s\>])(%s)(?=[\s\<\.,:])" % "|".join(keys),
            re.IGNORECASE | re.UNICODE | re.MULTILINE
        )
        return regex

    def handle_starttag(self, tag, attrs):
        super(SubHtml, self).handle_starttag(tag, attrs)
#        print "handle start tag: %r attrs: %r" % (tag, attrs)
        if tag in self.skip_tags:
            self.in_skip_tag = tag

    def handle_endtag(self, tag):
        super(SubHtml, self).handle_endtag(tag)
#        print "handle end tag: %r" % tag
        if tag == self.in_skip_tag:
            self.in_skip_tag = None

    def handle_data(self, data):
#        print "handle data: %r" % data
        if data and data.strip(" \n\t") and self.in_skip_tag is None:
            # data is not empty and we are not in a skip_tag area -> replace lexicon entries
            # call lexicon_data with the match object
            data = " %s " % data # work-a-round: http://www.python-forum.de/viewtopic.php?p=162915#162915 
            data = self.regex.sub(self.lexicon_data, data)
            data = data[1:-1] # work-a-round

        self.html += data



if __name__ == "__main__":
    import urllib2, time
    from pylucid_project.utils.diff import diff_lines

#    import doctest
#    doctest.testmod(verbose=False)
#    print "DocTest end."

    skip_tags = ('a', 'input', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'textarea', 'fieldset')

    class LexiconData(dict):
        def __call__(self, matchobject):
            word = matchobject.group(0)
            print "LexiconData.__call__: %r" % word
            term = self[word.lower()]
            return term + word + term

    lexicon_data = LexiconData({"foo bar": "1", "foo": "2", "bar": "3"})
#    lexicon_data = LexiconData({"match on nothing!": None})

#    url = "http://www.pylucid.org"
#    url = "http://www.google.com"
#    url = "http://www.python.org"
#    url = "http://www.heise.de"
#    url = "http://www.facebook.com"
#
#    print "request %r..." % url,
#    f = urllib2.urlopen(url)
#    html = f.read()
#    f.close()
#    print "OK"

#    html = '<a href="foo" title="here &quot;is a problem&quot; fuck">bar</a>'
    html = '''
<html><p><a href="Foo Bar"><strong>Foo Bar</strong> Bar</a>
one Foo Bar two FOO three BaR four
Here not: Fooo or XbarX
</p></html>
'''

    start_time = time.time()
    s = SubHtml(lexicon_data, skip_tags=["a"])
    s.feed(html)
    s.close()
    print "+++ duration: %.3fsec" % (time.time() - start_time)
    print diff_lines(html, s.html)

#    print "-" * 79
#    print html
#    print "-" * 79
#    print p.html
#    print "-" * 79





#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittests
    ~~~~~~~~~~~~~~~~~
    
    Info:
        - PyLucid initial data contains english and german pages.
        - related test in pylucid_plugins/language/tests.py
    
    :copyleft: 2010 by the django-weave team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"

from django.conf import settings
from django.http import HttpRequest
from django.contrib.auth.models import Group

from pylucid_project.tests.test_tools import basetest
from pylucid_project.apps.pylucid.models import Language
from pylucid_project.apps.pylucid.models import PageTree


class TestPagePermissions(basetest.BaseUnittest):
    """
    inherited from BaseUnittest:
        - initial data fixtures with default test users
        - self.login()
    """
    def _pre_setup(self, *args, **kwargs):
        super(TestPagePermissions, self)._pre_setup(*args, **kwargs)

        self.test_group = Group(name="test group")
        self.test_group.save()

    def _set_pagetree_permitViewGroup(self, url):
        """ set self.test_group top the PageTree object get from url """
        user = self._get_user("superuser")
        request = HttpRequest() # Create a fake request
        request.user = user
        (page_tree, _, _) = PageTree.objects.get_page_from_url(request, url)
        page_tree.permitViewGroup = self.test_group
        page_tree.save()

    def assertCanSee(self, response):
        self.assertResponse(response,
            must_contain=(
                '<title>PyLucid CMS - The buildin creole markup.</title>',
            ),
            must_not_contain=("Traceback",)
        )
        self.failUnlessEqual(response.status_code, 200)

    def test_no_permitViewGroup(self):
        """
        request a page, that is public
        """
        test_url = "example-pages/markups/creole"
        test_page_url = "/en/%s/" % test_url
        response = self.client.get(test_page_url)
        self.assertCanSee(response)

    def test_pagetree_permitViewGroup_1(self):
        """
        set a permitViewGroup on the last pagetree.
        """
        test_url = "example-pages/markups/creole"
        test_page_url = "/en/%s/" % test_url
        self._set_pagetree_permitViewGroup(test_url)
        response = self.client.get(test_page_url)
        self.assertPyLucidPermissionDenied(response)


if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
    management.call_command('test', __file__,
#        verbosity=1
        verbosity=0
    )



# coding: utf-8

"""
    PyLucid extrahead
    ~~~~~~~~~~~~~~~~~

    Stores extra html head content.
    A instance would be added to request.PYLUCID.extrahead in:
        pylucid.system.pylucid_objects
        
    Used in e.g.:
        pylucid_plugin.extrahead.context_middleware
        pylucid.shortcuts.render_pylucid_response

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate: $
    $Rev: $
    $Author: $

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


__version__= "$Rev:$"

import os
import inspect

from django.conf import settings


FILEPATH_SPLIT = "pylucid_project"

DEBUG_INFO = """\
<!-- extrahead from %(fileinfo)s - START -->
%(content)s
<!-- extrahead from %(fileinfo)s - END -->"""


class ExtraHead(list):
    """
    Simple store extra html head content from plugins.
    """
    def __init__(self, *args, **kwargs):
        super(ExtraHead, self).__init__(*args, **kwargs)
        
        if settings.DEBUG:
            # For self._get_fileinfo():
            self.basename = os.path.basename(__file__)
            if self.basename.endswith(".pyc"):
                # cut: ".pyc" -> ".py"
                self.basename = self.basename[:-1]
                    
            # Turn debug mode in JavaScript on
            list.append(self, DEBUG_INFO % {
                    "fileinfo": os.path.basename(__file__),
                    "content": '<script type="text/javascript">var debug=true;log("debug is on");</script>'
                }
            )            
        
    def append(self, content):
        """ add new content """
        content = content.strip()
        if settings.DEBUG:
            # Add debug info around content.
            fileinfo = self._get_fileinfo()
            content = DEBUG_INFO % {"fileinfo": fileinfo, "content": content}
            
        list.append(self, content)
        
    def get(self):
        """ return all extra head content """
        return "\n".join(self)
    
    def _get_fileinfo(self):
        """ return fileinfo: Where from the extra head content comes? """       
        try:    
            for stack_frame in inspect.stack():
                filepath = stack_frame[1]              
                lineno = stack_frame[2]

                # go forward in the stack, to outside of this file.
                if os.path.basename(filepath) != self.basename:
                    break

            filepath = "..." + filepath.split(FILEPATH_SPLIT,1)[1]
            return "%s line %s" % (filepath, lineno)
        except Exception, e:
            return "(inspect Error: %s)" % e



# coding: utf-8

"""
    PyLucid blog plugin
    ~~~~~~~~~~~~~~~~~~~

    A simple blog system.

    http://feedvalidator.org/
    
    TODO:
        * Detail view, use BlogEntry.get_absolute_url()

    :copyleft: 2008-2010 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""


from django import http
from django.conf import settings
from django.contrib import messages
from django.contrib.syndication.views import Feed
from django.utils.feedgenerator import Rss201rev2Feed, Atom1Feed
from django.utils.translation import ugettext as _
from django.views.decorators.csrf import csrf_protect

from pylucid_project.apps.pylucid.system import i18n
from pylucid_project.apps.pylucid.decorators import render_to
from pylucid_project.utils.safe_obtain import safe_pref_get_integer

from blog.models import BlogEntry
from blog.preference_forms import BlogPrefForm

# from django-tagging
from tagging.models import Tag, TaggedItem



def _add_breadcrumb(request, *args, **kwargs):
    """ shortcut for add breadcrumb link """
    context = request.PYLUCID.context
    breadcrumb_context_middlewares = context["context_middlewares"]["breadcrumb"]
    breadcrumb_context_middlewares.add_link(*args, **kwargs)


def _get_queryset(request, tags=None, filter_language=False):
    # Get all blog entries, that the current user can see
    queryset = BlogEntry.objects.all_accessible(request, filter_language=filter_language)

    if tags is not None:
        # filter by tags 
        queryset = TaggedItem.objects.get_by_model(queryset, tags)

    return queryset


def _split_tags(raw_tags):
    "simple split tags from url"
    tags = raw_tags.strip("/").split("/")
    return tags


class RssFeed(Feed):
    feed_type = Rss201rev2Feed
    filename = "feed.rss"
    title = _("Blog - RSS feed")
    link = "/"
    description_template = "blog/feed_description.html"

    def __init__(self, request, tags=None):
        self.request = request
        # client favored Language instance:
        lang_entry = request.PYLUCID.current_language
        self.language = lang_entry.code

        if tags is not None:
            tags = _split_tags(tags)
        self.tags = tags

        # Get max number of feed entries from request.GET["count"]
        # Validate/Limit it with information from DBPreferences 
        self.count, error = safe_pref_get_integer(
            request, "count", BlogPrefForm,
            default_key="initial_feed_count", default_fallback=5,
            min_key="initial_feed_count", min_fallback=5,
            max_key="max_feed_count", max_fallback=30
        )

    def description(self):
        if self.tags is None:
            return _("Last %s blog articles") % self.count
        else:
            return _(
                 "Last %(count)s blog articles tagged with: %(tags)s"
            ) % {"count":self.count, "tags": ",".join(self.tags)}

    def items(self):
        queryset = _get_queryset(self.request, self.tags, filter_language=True)
        return queryset[:self.count]

    def item_title(self, item):
        return item.headline

    def item_author_name(self, item):
        return item.createby

    def item_link(self, item):
        return item.get_absolute_url()


class AtomFeed(RssFeed):
    """
    http://docs.djangoproject.com/en/dev/ref/contrib/syndication/#publishing-atom-and-rss-feeds-in-tandem
    """
    feed_type = Atom1Feed
    filename = "feed.atom"
    title = _("Blog - Atom feed")
    subtitle = RssFeed.description


# The last class is the fallback class, if filename doesn't match
FEEDS = [AtomFeed, RssFeed]



@render_to("blog/summary.html")
def summary(request):
    """
    Display summary list with all blog entries.
    """
    # Get all blog entries, that the current user can see
    queryset = _get_queryset(request, filter_language=True)

    # Limit the queryset with django Paginator
    paginator = BlogEntry.objects.paginate(request, queryset)

    tag_cloud = BlogEntry.objects.get_tag_cloud(request)

    current_lang = request.PYLUCID.current_language.description
    _add_breadcrumb(request, _("All articles in %s.") % current_lang)

    # For adding page update information into context by pylucid context processor
    try:
        # Use the newest blog entry fro date info
        request.PYLUCID.updateinfo_object = queryset.latest("lastupdatetime")
    except BlogEntry.DoesNotExist:
        # No blog entries created, yet.
        pass

    context = {
        "entries": paginator,
        "tag_cloud": tag_cloud,
        "CSS_PLUGIN_CLASS_NAME": settings.PYLUCID.CSS_PLUGIN_CLASS_NAME,
        "feeds": FEEDS,
    }
    return context


@render_to("blog/summary.html")
def tag_view(request, tags):
    """
    Display summary list with blog entries filtered by the given tags.
    """
    tags = _split_tags(tags)

    # Get all blog entries, that the current user can see
    queryset = _get_queryset(request, tags, filter_language=True)

    # Limit the queryset with django Paginator
    paginator = BlogEntry.objects.paginate(request, queryset)

    # Add link to the breadcrumbs ;)
    text = _("All items tagged with: %s") % ", ".join(tags)
    current_lang = request.PYLUCID.current_language.description
    _add_breadcrumb(request, text, text + _(" and are written in %s.") % current_lang)

    tag_cloud = BlogEntry.objects.get_tag_cloud(request)

    context = {
        "entries": paginator,
        "tag_cloud": tag_cloud,
        "add_tag_filter_link": True, # Add + tag filter link
        "CSS_PLUGIN_CLASS_NAME": settings.PYLUCID.CSS_PLUGIN_CLASS_NAME,
        "used_tags": tags,
        "tags": "/".join(tags),
        "feeds": FEEDS,
    }
    return context


@csrf_protect
@render_to("blog/detail_view.html")
def detail_view(request, id, title):
    """
    Display one blog entry with a comment form.
    """
    # Get all blog entries, that the current user can see
    queryset = _get_queryset(request, filter_language=False)

    try:
        entry = queryset.get(pk=id)
    except BlogEntry.DoesNotExist:
        # It's possible that the user comes from a external link.
        msg = "Blog entry doesn't exist."
        if settings.DEBUG or request.user.is_staff:
            msg += " (ID %r wrong.)" % id
        messages.error(request, msg)
        return summary(request)

    new_url = i18n.assert_language(request, entry.language)
    if new_url:
        # the current language is not the same as entry language -> redirect to right url
        # e.g. someone followed a external link to this article, but his preferred language
        # is a other language as this article. Activate the article language and "reload"
        return http.HttpResponsePermanentRedirect(new_url)

    # Add link to the breadcrumbs ;)
    _add_breadcrumb(request, entry.headline, _("Article '%s'") % entry.headline)

    tag_cloud = BlogEntry.objects.get_tag_cloud(request)

    # Change permalink from the blog root page to this entry detail view
    permalink = entry.get_permalink(request)
    request.PYLUCID.context["page_permalink"] = permalink # for e.g. the HeadlineAnchor

    # Add comments in this view to the current blog entry and not to PageMeta
    request.PYLUCID.object2comment = entry

    # For adding page update information into context by pylucid context processor
    request.PYLUCID.updateinfo_object = entry

    context = {
        "page_title": entry.headline, # Change the global title with blog headline
        "entry": entry,
        "tag_cloud": tag_cloud,
        "CSS_PLUGIN_CLASS_NAME": settings.PYLUCID.CSS_PLUGIN_CLASS_NAME,
        "page_permalink": permalink, # Change the permalink in the global page template
    }
    return context

#------------------------------------------------------------------------------


@render_to("blog/select_feed.html")
def select_feed(request):
    """ Display a list with existing feed filenames. """
    context = {"feeds": FEEDS}
    return context


def feed(request, filename, tags=None):
    """
    return RSS/Atom feed for all blog entries and filtered by tags. 
    Feed format is selected by filename.
    """
    for feed_class in FEEDS:
        if filename == feed_class.filename:
            break

    # client favoured Language instance:
    lang_entry = request.PYLUCID.current_language

    # Work-a-round for http://code.djangoproject.com/ticket/13896
    old_lang_code = settings.LANGUAGE_CODE
    settings.LANGUAGE_CODE = lang_entry.code

    feed = feed_class(request, tags)
    response = feed(request)

    settings.LANGUAGE_CODE = old_lang_code

    return response




# coding: utf-8

from django import forms
from django.utils.translation import ugettext as _

from dbpreferences.forms import DBPreferencesBaseForm


class BreadcumbPrefForm(DBPreferencesBaseForm):
    separator = forms.CharField(
        initial = _(u" \u00BB "),
        help_text = _('Seperator between every breadcrumb link'),
    )
    print_last_page = forms.BooleanField(
        initial = True, required=False,
        help_text = _(
            "If checked the actual page will be the last page in the bar."
            " Otherwise the parentpage."
        ),
    )
    print_index = forms.BooleanField(
        initial = True, required=False,
        help_text = _('If checked every back link bar starts with a link to "index_url"'),
    )
    index_url = forms.CharField(
        initial = "/",
        help_text = _("The url used for print_index. Note: not verify if the url exists."),
    )
    index = forms.CharField(
        initial = _("Index"),
        help_text = _('the name that is printed for the indexpage'),
    )

    class Meta:
        app_label = 'breadcrumb'


# coding:utf-8

from django.db import models
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.managers import CurrentSiteManager

from pylucid_project.apps.pylucid.models.base_models import UpdateInfoBaseModel, BaseModel
from pylucid_project.apps.pylucid.models import Language


class UpdateJournal(BaseModel):
    """
    List of all last update data, used for creating the last update table.
    
    inherited attributes from UpdateInfoBaseModel:
        get_absolute_uri()
    """
    lastupdatetime = models.DateTimeField(auto_now=True, help_text="Time of the last change.",)
    user_name = models.CharField(_("user's name"), max_length=50, blank=True)

    objects = models.Manager()

    site = models.ForeignKey(Site, default=Site.objects.get_current)
    on_site = CurrentSiteManager()

    # Content-object field
    content_type = models.ForeignKey(
        ContentType, verbose_name=_('content type'), related_name="content_type_set_for_%(class)s"
    )
    object_url = models.URLField(verbose_name=_('object url'), help_text="absolute url to the item.",)

    language = models.ForeignKey(Language)

    title = models.CharField(blank=True, max_length=256,
        help_text="A long page title (for e.g. page title or link title text)"
    )
    staff_only = models.BooleanField(help_text="Viewable only by staff users?")

    def get_site(self):
        return self.site

    def get_absolute_url(self):
        return self.object_url

    def __unicode__(self):
        return u"Update Journal entry %r (id: %i, site: %s)" % (
            self.object_url, self.id, self.site.domain
        )

    class Meta:
        verbose_name = 'Update Journal entry'
        verbose_name_plural = 'Update Journal entries'
        ordering = ("-lastupdatetime",)


class PageUpdateListObjectsManager(models.Manager):
    def add_entry(self, model_instance):
        """
        Add a new update journal entry.
        Called by signal handler in update_journal/__init__.py
        """
        update_info = model_instance.get_update_info()
        if update_info == None:
            # This entry should not be inserted in the update journal
            return

        # delete same entries
        object_url = update_info["object_url"]
        UpdateJournal.on_site.all().filter(object_url=object_url).delete()

        if isinstance(update_info["user_name"], User):
            user = update_info["user_name"]
            user_name = user.get_full_name() or user.username
            update_info["user_name"] = user_name

        if update_info["user_name"] == None:
            update_info["user_name"] = ""

        content_type = ContentType.objects.get_for_model(model_instance)
        instance, created = PageUpdateListObjects.on_site.get_or_create(content_type=content_type)
        update_info.update({
            "staff_only": instance.staff_only,
            "content_type": content_type,
        })
        UpdateJournal(**update_info).save()


class PageUpdateListObjects(UpdateInfoBaseModel):
    """ Witch objects should be listed in the last update table? """
    objects = PageUpdateListObjectsManager()

    site = models.ForeignKey(Site, default=Site.objects.get_current)
    on_site = CurrentSiteManager()

    content_type = models.ForeignKey(
        ContentType, verbose_name=_('content type'), related_name="content_type_set_for_%(class)s"
    )
    staff_only = models.BooleanField(default=False, help_text="Viewable only by staff users?")



# coding:utf-8

"""
    translate a PageContent.
"""

from django import http
from django.conf import settings
from django.contrib import messages
from django.db import transaction
from django.utils.translation import ugettext_lazy as _

from pylucid_project.apps.pylucid.decorators import check_permissions, render_to
from pylucid_project.apps.pylucid.markup.converter import apply_markup
from pylucid_project.apps.pylucid.models import PageTree, PageMeta, PageContent, Language
from pylucid_project.utils.translate import translate

from page_admin.forms import PageMetaForm, PageContentForm, LanguageSelectForm



def _select_language(request, context, source_pagemeta):
    """
    choose the translation destination language.
    If one one other language available -> return it directly.
    """
    other_languages = Language.objects.exclude(code=source_pagemeta.language.code)
    if len(other_languages) == 0:
        # should not happend
        messages.error(request, "Error: There exist only one Language!")
        return http.HttpResponseRedirect(source_pagemeta.get_absolute_url())
    elif len(other_languages) == 1:
        # Only one other language available, so the user must not choose one ;)
        return other_languages[0]
    else:
        # There are more than one other language -> display a form for choose one.
        if "language" in request.GET:
            form = LanguageSelectForm(other_languages, request.GET)
            if form.is_valid():
                lang_code = form.cleaned_data["language"]
                for lang in other_languages:
                    if lang.code == lang_code:
                        return lang
                raise RuntimeError() # should never happen
        else:
            default_lang_entry = Language.objects.get_or_create_default(request)
            form = LanguageSelectForm(other_languages, initial={
                    "language": default_lang_entry.code, # FIXME: Seems not to work
                })
        context.update({
            "title": _("Translate page '%(name)s' (%(desc_lang)s) - Select destination language") % {
                "name": source_pagemeta.name,
                "desc_lang": source_pagemeta.language.description
            },
            "template_name": "page_admin/select_translate_language.html",
            "form": form,
        })
        return context





@check_permissions(superuser_only=False, permissions=(
    "pylucid.add_pagecontent", "pylucid.add_pagemeta",
    "pylucid.change_pagecontent", "pylucid.change_pagemeta",
))
@render_to()
def translate_page(request, pagemeta_id=None):
    if not pagemeta_id:
        raise

    source_pagemeta = PageMeta.objects.get(id=pagemeta_id)
    pagetree = source_pagemeta.pagetree
    source_language = source_pagemeta.language

    is_pluginpage = pagetree.page_type == PageTree.PLUGIN_TYPE
    if is_pluginpage:
        messages.error(request, "TODO: Translate a plugin page!")
        return http.HttpResponseRedirect(source_pagemeta.get_absolute_url())
    else:
        source_pagecontent = PageContent.objects.get(pagemeta=source_pagemeta)

    context = {
        "abort_url": source_pagemeta.get_absolute_url(),
    }

    # select the destination languare
    result = _select_language(request, context, source_pagemeta)
    if isinstance(result, Language):
        # Language was selected or they exit only one other language
        dest_language = result
    elif isinstance(result, dict):
        # template context returned -> render language select form
        return result
    elif isinstance(result, http.HttpResponse):
        # e.g. error
        return result
    else:
        raise RuntimeError() # Should never happen


    context.update({
        "title": _("Translate page '%(name)s' (%(source_lang)s) into %(dest_lang)s.") % {
            "name": source_pagemeta.name,
            "source_lang": source_pagemeta.language.description,
            "dest_lang": dest_language.description,
        },
        "template_name": "page_admin/translate_page.html",
    })

    try:
        dest_pagemeta = PageMeta.objects.get(pagetree=pagetree, language=dest_language)
    except PageMeta.DoesNotExist:
        dest_pagemeta = None
    else:
        dest_pagecontent = PageContent.objects.get(pagemeta=dest_pagemeta)

    if request.method == "POST":
        source_pagemeta_form = PageMetaForm(
            request.POST, prefix="source", instance=source_pagemeta
        )
        source_pagecontent_form = PageContentForm(
            request.POST, prefix="source", instance=source_pagecontent
        )
        if dest_pagemeta is None:
            dest_pagemeta_form = PageMetaForm(request.POST, prefix="dest")
            dest_pagecontent_form = PageContentForm(request.POST, prefix="dest")
        else:
            dest_pagemeta_form = PageMetaForm(
                request.POST, prefix=dest_language.code, instance=dest_pagemeta
            )
            dest_pagecontent_form = PageContentForm(
                request.POST, prefix=dest_language.code, instance=dest_pagecontent
            )

        if not (source_pagemeta_form.is_valid() and source_pagecontent_form.is_valid() and
                            dest_pagemeta_form.is_valid() and dest_pagecontent_form.is_valid()):
            context["has_errors"] = True
        else:
            context["has_errors"] = False
            if "preview" in request.POST:
                context["source_preview"] = apply_markup(
                    source_pagecontent_form.cleaned_data["content"],
                    source_pagecontent_form.cleaned_data["markup"],
                    request, escape_django_tags=True
                )
                context["dest_preview"] = apply_markup(
                    dest_pagecontent_form.cleaned_data["content"],
                    dest_pagecontent_form.cleaned_data["markup"],
                    request, escape_django_tags=True
                )
                context["has_errors"] = False
            else: # All forms are valid and it's not a preview -> Save all.
                sid = transaction.savepoint()
                try:
                    source_pagecontent_form.save()
                    source_pagemeta_form.save()

                    # Create new PageMeta entry
                    new_pagemeta = dest_pagemeta_form.save(commit=False)
                    new_pagemeta.pagetree = pagetree
                    new_pagemeta.language = dest_language
                    new_pagemeta.save()

                    # Create new PageContent entry
                    new_pagecontent = dest_pagecontent_form.save(commit=False)
                    new_pagecontent.pagemeta = new_pagemeta
                    new_pagecontent.save()
                except:
                    transaction.savepoint_rollback(sid)
                    raise
                else:
                    transaction.savepoint_commit(sid)
                    if dest_pagemeta is None:
                        messages.info(request, "New content %r crerated." % new_pagecontent)
                    else:
                        messages.info(request, "All updated.")
                    return http.HttpResponseRedirect(new_pagemeta.get_absolute_url())
    else:
        context["has_errors"] = False
        source_pagemeta_form = PageMetaForm(
            prefix="source", instance=source_pagemeta
        )
        source_pagecontent_form = PageContentForm(
            prefix="source", instance=source_pagecontent
        )
        if dest_pagemeta is None:
            dest_pagemeta_form = PageMetaForm(
                prefix="dest", initial={
                    "robots": source_pagemeta.robots,
                    "permitViewGroup": source_pagemeta.permitViewGroup, # FIXME: Doesn't work
                }
            )
            dest_pagecontent_form = PageContentForm(
                prefix="dest", initial={
                    "markup": source_pagecontent.markup,
                }
            )
        else:
            dest_pagemeta_form = PageMetaForm(
                prefix=dest_language.code, instance=dest_pagemeta
            )
            dest_pagecontent_form = PageContentForm(
                prefix=dest_language.code, instance=dest_pagecontent
            )


        if "prefill" in request.GET:
            filled_fields = []
            if "content" not in dest_pagecontent_form.initial:
                source_content = source_pagecontent_form.initial["content"]
                try:
                    translated_content = translate(
                        source_content, src=source_language.code, to=dest_language.code
                    )
                except ValueError, err:
                    messages.info(request, "Can't translate content with google: %s" % err)
                else:
                    dest_pagecontent_form.initial["content"] = translated_content
                    filled_fields.append("content")
                    dest_pagecontent_form.fields['content'].widget.attrs['class'] = 'auto_translated'

            for key, source_value in source_pagemeta_form.initial.iteritems():
                if not source_value \
                    or not isinstance(source_value, basestring)\
                    or key == "robots" \
                    or dest_pagemeta_form.initial.get(key, None):
                    # Skip empty, non string, robots field and if dest. value exist
                    continue

                try:
                    dest_value = translate(
                        source_value, src=source_language.code, to=dest_language.code
                    )
                except ValueError, err:
                    messages.info(request,
                        "Can't translate %(key)s with google: %(err)s" % {"key":key, "err":err}
                    )
                else:
                    dest_pagemeta_form.initial[key] = dest_value
                    filled_fields.append(key)
                    dest_pagemeta_form.fields[key].widget.attrs['class'] = 'auto_translated'

            if filled_fields:
                messages.info(request, "These fields are translated via google: %s" % ", ".join(filled_fields))
            else:
                messages.info(request, "No fields translated via google.")

    source_pagecontent_form.language = source_language
    dest_pagecontent_form.language = dest_language

    pagemeta_fields = []
    for source_field, dest_field in zip(source_pagemeta_form, dest_pagemeta_form):
        source_field.language = source_language
        pagemeta_fields.append(source_field)
        dest_field.language = dest_language
        pagemeta_fields.append(dest_field)

    context.update({
        "form_url": request.path,
        "abort_url": source_pagemeta.get_absolute_url(),
        "all_forms": [
            source_pagemeta_form, source_pagecontent_form,
            dest_pagemeta_form, dest_pagecontent_form
        ],
        "source_pagemeta_form": source_pagemeta_form,
        "source_pagecontent_form": source_pagecontent_form,
        "dest_pagemeta_form": dest_pagemeta_form,
        "dest_pagecontent_form": dest_pagecontent_form,

        "pagemeta_fields": pagemeta_fields,
    })
    return context





# coding: utf-8

def save_receiver(sender, **kwargs):
    """
    signal post save handler -> create a new UpdateData entry
    connected with PageContent in: pylucid_project/apps/pylucid/models/__init__.py
    """
    model_instance = kwargs["instance"]
    from models import PageUpdateListObjects
    PageUpdateListObjects.objects.add_entry(model_instance)



# coding: utf-8

"""
    Gallery PyLucid Plugin - admin 
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Register model in django admin interface.

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author$

    :copyleft: 2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from django.http import HttpResponseRedirect
from django.contrib import admin
from django import forms

from pylucid_project.apps.pylucid.models import PageTree, PluginPage
from pylucid_project.apps.pylucid.base_admin import BaseAdmin, RedirectToPageAdmin

from gallery.models import GalleryModel



class GalleryAdminForm(forms.ModelForm):
    """
    Filter pagetree selection.
    Add only pagetree items witch are a gallery plugin page.
    """
    class Meta:
        model = GalleryModel

    def __init__(self, *args, **kwargs):
        super(GalleryAdminForm, self).__init__(*args, **kwargs)

        plugin_pages = PluginPage.objects.filter(app_label="pylucid_project.pylucid_plugins.gallery")
        # TODO: Filter pagetree's witch has already a Gallery PluginPage
        choices = [
            (page.pagetree.id, page.pagetree.get_absolute_url())
            for page in plugin_pages
        ]
        self.fields["pagetree"].choices = choices


class GalleryModelAdmin(RedirectToPageAdmin, BaseAdmin):
    form = GalleryAdminForm
    list_display = (
        "view_on_site_link", "path", "template",
        "lastupdatetime", "lastupdateby"
    )
    list_display_links = ("path",)
    list_filter = ("template", "createby", "lastupdateby",)
    date_hierarchy = 'lastupdatetime'
    #search_fields = ("path", "template",)

admin.site.register(GalleryModel, GalleryModelAdmin)



# -*- coding: utf-8 -*-

"""
    PyLucid sub_menu plugin
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    Generates a link list of all sub pages.

    Last commit info:
    ~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author$

    :copyleft: 2005-2009 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details
"""

__version__ = "$Rev$"


from pylucid_project.apps.pylucid.models import PageTree, PageMeta, Language
from pylucid_project.apps.pylucid.decorators import render_to


@render_to("sub_menu/sub_menu.html")
def lucidTag(request):
    """ build the sub menu with all existing sub pages"""
    pagetree = request.PYLUCID.pagetree # current models.PageContent instance
    current_lang = request.PYLUCID.current_language # Client prefered language
    default_lang = Language.objects.get_or_create_default(request) # System default language

    # Build a PageTree ID list of all accessible subpages
    queryset = PageTree.objects.all_accessible(request.user, filter_showlinks=True)
    queryset = queryset.filter(parent=pagetree)
    sub_pagetree_ids = queryset.values_list("id", flat=True)

    # Get PageMeta in client prefered language
    queryset = PageMeta.objects.filter(language=current_lang)
    pagemeta1 = queryset.filter(pagetree__in=sub_pagetree_ids).select_related()
    existing_pagetree1 = [pagemeta.pagetree.id for pagemeta in pagemeta1]

    # Build a list of all PageTree IDs witch haven't a PageMeta
    missing_pagetree1 = [id for id in sub_pagetree_ids if id not in existing_pagetree1]
    if not missing_pagetree1:
        # No missing languages -> Use the source queryset
        sub_pages = list(pagemeta1)
    else: # Some pages didn't exist in current language -> use system default language
        queryset = PageMeta.objects.filter(language=default_lang)
        pagemeta2 = queryset.filter(pagetree__in=missing_pagetree1).select_related()

        # PageMeta in current language + PageMeta in system default language
        sub_pages = list(pagemeta1) + list(pagemeta2)

        # Build a list of existing PageMeta
        existing_pagetree2 = [pagemeta.pagetree.id for pagemeta in sub_pages]
        # Exist there pages not in Client prefered language and not in System default language?
        missing_pagetree2 = [id for id in sub_pagetree_ids if id not in existing_pagetree2]
        if missing_pagetree2: # There are still missing PageMeta.
            queryset = PageMeta.objects.exclude(language=current_lang).exclude(language=default_lang)
            sub_pages3 = queryset.filter(pagetree__in=missing_pagetree2).select_related()

            # merge sub pages
            sub_pages += list(sub_pages3)

    # sort by PageTree positions 
    sub_pages.sort(cmp=lambda x, y: cmp(x.pagetree.position, y.pagetree.position))

    return {"sub_pages": sub_pages}














# coding: utf-8

from django import forms
from django.utils.translation import ugettext_lazy as _

from pylucid_project.utils.site_utils import SitePreselectPreference

from dbpreferences.forms import DBPreferencesBaseForm


class BlogPrefForm(SitePreselectPreference, DBPreferencesBaseForm):
    max_anonym_count = forms.IntegerField(
        initial=10, min_value=1,
        help_text=_(
            "The maximal numbers of blog entries, displayed together"
            " for anonymous users"
        ),
    )
    max_user_count = forms.IntegerField(
        initial=30, min_value=1,
        help_text=_(
            "The maximal numbers of blog entries, displayed together"
            " for logged in PyLucid users."
        ),
    )
    
    initial_feed_count = forms.IntegerField(
        initial=5, min_value=1,
        help_text=_("Default numbers of blog articles in RSS/Atom feed."),
    )
    max_feed_count = forms.IntegerField(
        initial=30, min_value=1,
        help_text=_("The maximal numbers of blog articles in RSS/Atom feed."),
    )
    class Meta:
        app_label = 'blog'



# -*- coding: utf-8 -*-

"""
    SimpleStringIO
    ~~~~~~~~~~~~~~

    Last commit info:
    ~~~~~~~~~~~~~~~~~
    $LastChangedDate$
    $Rev$
    $Author$

    :copyleft: 2007-2008 by the PyLucid team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

if __name__ == "__main__":
    # For doctest only
    import os
    os.environ["DJANGO_SETTINGS_MODULE"] = "django.conf.global_settings"

from django.conf import settings


class SimpleStringIO(object):
    """
    Minimalistic StringIO-file-like object.
    Encode unicode to the default charset (imported from the settings).
    See http://docs.python.org/lib/bltin-file-objects.html
    
    >>> s = SimpleStringIO()
    >>> s.write("one\\n")
    >>> s(u"two")
    >>> s.getvalue()
    'one\\ntwo'
    >>> s.getlines()
    ['one\n', 'two']
    """
    def __init__(self):
        self._charset = settings.DEFAULT_CHARSET
        self._container = []

    def write(self, content):
        """
        Append a new chunk.
        Encode unicode to the default charset.
        """
        if isinstance(content, unicode):
            content = content.encode(self._charset)
        self._container.append(content)

    def __call__(self, content):
        self.write(content)

    def getlines(self):
        """
        returns the container list
        """
        return self._container

    def getvalue(self):
        """
        Get all content.
        """
        content = ''.join(self._container)
        return content

    def isatty(self):
        """
        Used for the _install section: Redirected the syncdb command.
        It checks sys.stdout.isatty() in django.core.management.color
        """
        return False


if __name__ == "__main__":
    import doctest
    doctest.testmod(
#        verbose=True
        verbose=False
    )
    print "DocTest end."


#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittests
    ~~~~~~~~~~~~~~~~~
    
    Info:
        - PyLucid initial data contains english and german pages.
        - There exist only "PyLucid CMS" lexicon entry in english and german
    
    :copyleft: 2010 by the django-weave team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os


if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"


from django.conf import settings
from django.core.urlresolvers import reverse

from pylucid_project.tests.test_tools import basetest
from pylucid_project.apps.pylucid.models import PageTree


class RedirectPluginTest(basetest.BaseLanguageTestCase):
    def _pre_setup(self, *args, **kwargs):
        """ create some language related attributes """
        super(RedirectPluginTest, self)._pre_setup(*args, **kwargs)

        self.new_plugin_page_url = reverse("PageAdmin-new_plugin_page")
        self.redirectmodel_add_url = reverse("admin:redirect_redirectmodel_add")

        self.login("superuser")

    def test_add_form(self):
        response = self.client.get(self.new_plugin_page_url)
        self.assertResponse(response,
            must_contain=(
                'Create a new plugin page',
                '<option value="pylucid_project.pylucid_plugins.redirect">pylucid_plugins.redirect</option>',
            ),
            must_not_contain=("Traceback",)
        )

    def test_create_plugin_page(self):
        """
        Create a 'redirect' plugin page and check the redirection after this.
        """
        test_slug = "test_redirect"
        destination_url = "http://github.com/jedie/PyLucid"

        response = self.client.post(self.new_plugin_page_url,
            data={'app_label': 'pylucid_project.pylucid_plugins.redirect',
            'design': 1,
            'position': 0,
            'slug': test_slug,
            'urls_filename': 'urls.py'
            }
        )
        url = "http://testserver/en/%s/" % test_slug
        self.assertRedirect(response, url, status_code=302)

        response = self.client.get(url)
        self.assertRedirect(response, "http://testserver" + self.redirectmodel_add_url, status_code=302)

        response = self.client.get(self.redirectmodel_add_url)
        self.assertResponse(response,
            must_contain=(
                "Redirect entry for page: /%s/ doesn&#39;t exist, please create." % test_slug,
                'Add redirect model',
            ),
            must_not_contain=("Traceback",)
        )

        page_tree_id = PageTree.on_site.get(slug=test_slug).id
        response = self.client.post(self.redirectmodel_add_url,
            data={
            "pagetree": page_tree_id,
            "destination_url": destination_url,
            "response_type": 302,
            }
        )
        self.assertRedirect(response,
            "http://testserver" + reverse("admin:redirect_redirectmodel_changelist"),
            status_code=302
        )

        response = self.client.get(url)
        self.assertRedirect(response, destination_url, status_code=302)



if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
#    management.call_command('test', "pylucid_plugins.lexicon.tests.RedirectPluginTest", verbosity=0)
    management.call_command('test', __file__, verbosity=1)



#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittests
    ~~~~~~~~~~~~~~~~~
    
    Info:
        - PyLucid initial data contains english and german pages.
        - There exist only "PyLucid CMS" blog entry in english and german
    
    :copyleft: 2010 by the django-weave team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"

from django.conf import settings
from django.contrib.comments.forms import CommentForm
from django.contrib.comments.models import Comment
from django.core import mail
from django.test.client import Client

from django_tools.unittest_utils.BrowserDebug import debug_response

from pylucid_project.tests.test_tools import basetest
from pylucid_project.apps.pylucid.models import PageMeta
from pylucid_comments.views import _get_preferences

class PyLucidCommentsTestCase(basetest.BaseUnittest):
    def getValidData(self, obj, **kwargs):
        """ work-a-round for comment form security fields """
        f = CommentForm(obj)
        d = {
            'name'      : 'John Doe',
            'email'     : 'john.doe@example.tld',
            'url'       : '',
            'comment'   : 'This is my comment',
        }
        d.update(f.initial)
        d.update(kwargs)
        return d
        

class PyLucidCommentsPageMetaTest(PyLucidCommentsTestCase):
    
    def _pre_setup(self, *args, **kwargs):
        super(PyLucidCommentsPageMetaTest, self)._pre_setup(*args, **kwargs)
        self.pagemeta = PageMeta.on_site.all()[0]
        self.absolute_url = self.pagemeta.get_absolute_url()
        
    def setUp(self):
        Comment.objects.all().delete()
        
    def _get_form(self):
        url = self.absolute_url + "?pylucid_comments=get_form"
        data = self.getValidData(self.pagemeta)
        response = self.client.post(url,
            {
                "content_type": data["content_type"],
                "object_pk": data["object_pk"]
            },
            HTTP_X_REQUESTED_WITH='XMLHttpRequest'
        )
        return response
    
    def test_get_form(self):
        """ get the comment form via AJAX """
        response = self._get_form()
        self.assertResponse(response,
            must_contain=(
                '<form action="JavaScript:void(0)" method="post" id="comment_form">',
                '<input type="hidden" name="content_type" value="pylucid.pagemeta"',
                '<input type="hidden" name="object_pk" value="%i"' % self.pagemeta.pk,
                '<input checked="checked" type="checkbox" name="notify" id="id_notify" />'
            ),
            must_not_contain=(
                "Traceback", "Form errors", "field is required",
                "<body", "</html>"
            )
        )
    
    def test_submit_comment(self):
        settings.DEBUG = True # Display a comment error page
        self.failUnless(Comment.objects.count() == 0)
        self.failUnless(len(mail.outbox) == 0, len(mail.outbox))
        url = self.absolute_url + "?pylucid_comments=submit"
        
        # submit a valid comments form
        data = self.getValidData(self.pagemeta, comment="from test_submit_comment()")
        response = self.client.post(url, data, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        
        # Check if comment created
        self.failUnless(Comment.objects.count() == 1)
        
        # Check if page should reload (JavaScript do this)
        self.failUnlessEqual(response.content, 'reload')
        
        # Check if ADMINS get's a email.
        #for email in mail.outbox:print email.message()
        if len(mail.outbox) > 1:
            print "FIXME: Why two mails sended???"
#        self.failUnless(len(mail.outbox) == 1, len(mail.outbox)) # FIXME: Why two mails sended???
        self.failUnless(len(mail.outbox) > 0, len(mail.outbox))
        email_text = mail.outbox[0].message()
        #print email_text
        self.failUnless("The comment is public." not in email_text)
        self.failUnless(data["name"] not in email_text)
        self.failUnless(data["comment"] not in email_text)
        self.failUnless("http://testserver%s" % self.absolute_url not in email_text)
        
        # Check if anonymous data saved in a cookie, for later usage
        self.failUnless("comments_data" in response.cookies)
        comments_data = response.cookies["comments_data"].value
        self.failUnless(comments_data.startswith("John Doe;john.doe@example.tld;"))
        
        # Check if anonymous data stored in cookie would be used:
        response = self._get_form()
        self.assertResponse(response,
            must_contain=(
                '<input id="id_name" type="text" name="name" value="John Doe" maxlength="50" />',
                '<input type="text" name="email" value="john.doe@example.tld" id="id_email" />',
            )
        )
        
    def test_submit_preview(self):
        settings.DEBUG = True # Display a comment error page
        self.failUnless(Comment.objects.count() == 0)
        url = self.absolute_url + "?pylucid_comments=submit"
        data = self.getValidData(self.pagemeta, preview="On", comment="comment from test_submit_preview()")        
        response = self.client.post(url, data, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertResponse(response,
            must_contain=(
                'Preview your comment',
                '<blockquote><p>comment from test_submit_preview()</p></blockquote>',
                '<form action="JavaScript:void(0)" method="post" id="comment_form">',
                '<input type="hidden" name="content_type" value="pylucid.pagemeta"',
                '<input type="hidden" name="object_pk" value="%i"' % self.pagemeta.pk,
                'type="checkbox" name="notify" id="id_notify" />',
            ),
            must_not_contain=(
                "Traceback", "Form errors", "field is required",
                "<body", "</html>"
            )
        )
    
    def test_submit_no_comment(self):
        settings.DEBUG = True # Display a comment error page
        self.failUnless(Comment.objects.count() == 0)
        url = self.absolute_url + "?pylucid_comments=submit"
        data = self.getValidData(self.pagemeta, comment="")
        response = self.client.post(url, data, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.failUnless(Comment.objects.count() == 0)
        self.assertResponse(response,
            must_contain=(
                'Please correct the error below',
                'This field is required.',
                '<form action="JavaScript:void(0)" method="post" id="comment_form">',
                '<input type="hidden" name="content_type" value="pylucid.pagemeta"',
                '<input type="hidden" name="object_pk" value="%i"' % self.pagemeta.pk,
                'type="checkbox" name="notify" id="id_notify" />',
            ),
            must_not_contain=("Traceback", "<body", "</html>")
        )
    
    def test_submit_spam(self):
        settings.DEBUG = True # Display a comment error page
        self.failUnless(Comment.objects.count() == 0)
        url = self.absolute_url + "?pylucid_comments=submit"
        data = self.getValidData(self.pagemeta,
            comment="Penis enlargement pills: http://en.wikipedia.org/wiki/Penis_enlargement ;)"
        )
        response = self.client.post(url, data, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        
        # Check if page should reload (JavaScript do this)
        self.failUnlessEqual(response.content, 'reload')

        # Check if ADMINS get's a email.
        #for email in mail.outbox:print email.message()
        if len(mail.outbox) > 1:
            print "FIXME: Why two mails sended???"
#        self.failUnless(len(mail.outbox) == 1, len(mail.outbox)) 
        self.failUnless(len(mail.outbox) > 0, len(mail.outbox))
        email_text = mail.outbox[0].message()
        #print email_text
        self.failUnless("The comment is public." not in email_text)
        self.failUnless(data["name"] not in email_text)
        self.failUnless(data["comment"] not in email_text)
        self.failUnless("http://testserver%s" % self.absolute_url not in email_text)

        # Check if comment created
        self.failUnless(Comment.objects.count() == 1)
        comment = Comment.objects.all()[0]
        self.failUnless(comment.is_public == False)
        
        # 'Reload' and check page message
        response = self.client.get(self.absolute_url)
        self.assertResponse(response,
            must_contain=(
                'Your comment waits for moderation.',
            ),
            must_not_contain=(
                "Traceback", "Form errors", "field is required",
                "Penis", "enlargement", "pills",
            )
        )
        
    def test_DOS_attack(self):
        settings.DEBUG = True # Display a comment error page
        self.failUnless(Comment.objects.count() == 0)
        url = self.absolute_url + "?pylucid_comments=submit"
              
        preferences = _get_preferences()
        ban_limit = preferences["ban_limit"]
        
        # Hold if all three events would been received. 
        tested_under_limit = False
        tested_limit_reached = False
        tested_banned = False
        
        for no in xrange(1, ban_limit+2):          
            # submit a valid comments form
            data = self.getValidData(self.pagemeta, comment="test_DOS_attack() comment no %i" % no)
            response = self.client.post(url, data, HTTP_X_REQUESTED_WITH='XMLHttpRequest')
            
            if no>ban_limit:
                # IP is on the ban list
                tested_banned = True
                self.assertStatusCode(response, 403) # get forbidden page
                comment_count = Comment.objects.count()
                self.failUnless(comment_count == ban_limit-1)
            elif no==ban_limit:
                # The limit has been reached
                tested_limit_reached = True
                self.assertResponse(response, must_contain=('Add IP to ban list.',))
                comment_count = Comment.objects.count()
                self.failUnless(comment_count == ban_limit-1)
            else:
                # under ban limit: comment was saved, page should be reloaded
                tested_under_limit = True
                self.assertResponse(response,
                    must_contain=('reload',),
                    must_not_contain=(
                        "Traceback", "Form errors", "field is required",
                        "<!DOCTYPE", "<body", "</html>",
                    )
                )
                comment_count = Comment.objects.count()
                self.failUnless(comment_count == no)

        # Check if all three events have been received.
        self.failUnless(tested_limit_reached == True)
        self.failUnless(tested_under_limit == True)
        self.failUnless(tested_banned == True)



if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
#    management.call_command('test', "pylucid_plugins.blog.tests.BlogPluginArticleTest",
##        verbosity=0,
#        verbosity=1,
#        failfast=True
#    )
    management.call_command('test', __file__,
        verbosity=1,
#        verbosity=0,
#        failfast=True
    )



#!/usr/bin/env python
# coding: utf-8

"""
    PyLucid unittests
    ~~~~~~~~~~~~~~~~~
    
    :copyleft: 2010 by the django-weave team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

import os

if __name__ == "__main__":
    # run all unittest directly
    os.environ['DJANGO_SETTINGS_MODULE'] = "pylucid_project.settings"

from django.conf import settings
from django.test.client import Client
from django.core.urlresolvers import reverse

from pylucid_project.tests.test_tools import basetest
from pylucid_project.apps.pylucid.models import PluginPage, PageContent



class FindReplaceTest(basetest.BaseLanguageTestCase):
    """
    inherited from BaseUnittest:
        - assertPyLucidPermissionDenied()
        - initial data fixtures with default test users
        - self.login()
    
    inherited from BaseLanguageTest:
        - self.default_language - system default Language model instance (default: en instance)
        - self.other_lang_code - alternative language code than system default (default: 'de')
        - self.other_language - alternative Language mode instance (default: de instance)
        - assertContentLanguage() - Check if response is in right language
    """
    def test_replace_form(self):
        """ Simply test if we get the find&replace form as a superuser. """
        self.login("superuser")
        url = reverse("FindAndReplace-find_and_replace")
        response = self.client.get(url)
        self.assertResponse(response,
            must_contain=(
                '<form action="%s" method="post" id="find_and_replace' % url,
                '<input type="text" name="find_string"',
                '<input type="text" name="replace_string"',
            ),
            must_not_contain=("Traceback", "XXX INVALID TEMPLATE STRING")
        )

    def test_pagecontent_replace_simulate(self):
        self.login("superuser")
        url = reverse("FindAndReplace-find_and_replace")
        response = self.client.post(url, data={
            'find_string': 'the',
            'replace_string': 'XXX',
            'content_type': 0, # PageContent
            'languages': ['de', 'en'],
            'save': 'find and replace',
            'sites': ['1'],
            'simulate': 'on'
        })
        self.assertResponse(response,
            must_contain=(
                '<link rel="stylesheet" type="text/css" href="/media/PyLucid/headfile_cache/pygments.css"',
                '<form action="%s" method="post" id="find_and_replace' % url,
                '<input type="text" name="find_string"',
                '<input type="text" name="replace_string"',
                'Simulate only, no entry changed.',
                '<legend class="pygments_code">Diff</legend>',
                '<span class="gd">-',
                '<span class="gi">+',
                '?  ', ' ^^^ ',
            ),
            must_not_contain=("Traceback", "XXX INVALID TEMPLATE STRING")
        )

    def test_pagecontent_replace(self):
        self.login("superuser")
        url = reverse("FindAndReplace-find_and_replace")
        response = self.client.post(url, data={
            'find_string': 'Welcome to your fesh PyLucid CMS installation',
            'replace_string': 'XXX replaced XXX',
            'content_type': 0, # PageContent
            'languages': ['de', 'en'],
            'sites': ['1'],
            'save': 'find and replace',
        })
        self.assertResponse(response,
            must_contain=(
                '<link rel="stylesheet" type="text/css" href="/media/PyLucid/headfile_cache/pygments.css"',
                '<form action="%s" method="post" id="find_and_replace' % url,
                '<input type="text" name="find_string"',
                '<input type="text" name="replace_string"',
                '<legend class="pygments_code">Diff</legend>',
                '<span class="gd">- Welcome to your fesh PyLucid CMS installation ;)</span>',
                '<span class="gi">+ XXX replaced XXX ;)</span>',
            ),
            must_not_contain=("Traceback", 'Simulate only, no entry changed.',)
        )
        response = self.client.get("/")
        self.assertResponse(response,
            must_contain=('XXX replaced XXX',),
            must_not_contain=(
                "Traceback", 'Welcome to your fesh PyLucid CMS installation',
                "XXX INVALID TEMPLATE STRING"
            )
        )



if __name__ == "__main__":
    # Run all unittest directly
    from django.core import management
    management.call_command('test', __file__,
        verbosity=1,
#        verbosity=0,
#        failfast=True
    )
